@@ -1,6 +1,6 @@
 #!/bin/sh
 
-executable="nexuiz-osx-ppc-dedicated"
+executable="nexuiz-osx-dedicated"
 
 cd "`dirname \"${0}\"`"
 exec ./${executable} -game pro +exec server.cfg "${@}"
