@@ -513,7 +513,7 @@ void CL_UpdateRenderEntity(entity_render_t *ent)
 {
 	vec3_t org;
 	vec_t scale;
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	// update the inverse matrix for the renderer
 	Matrix4x4_Invert_Simple(&ent->inversematrix, &ent->matrix);
 	// update the animation blend state
@@ -864,7 +864,7 @@ void CL_UpdateNetworkEntity(entity_t *e, int recursionlimit, qboolean interpolat
 	int j, k, l, frame;
 	float origin[3], angles[3], delta[3], lerp, d;
 	entity_t *t;
-	model_t *model;
+	dp_model_t *model;
 	//entity_persistent_t *p = &e->persistent;
 	//entity_render_t *r = &e->render;
 	// skip inactive entities and world
@@ -946,11 +946,7 @@ void CL_UpdateNetworkEntity(entity_t *e, int recursionlimit, qboolean interpolat
 		interpolate = false;
 	if (e == cl.entities + cl.playerentity && cl.movement_predicted && (!cl.fixangle[1] || !cl.fixangle[0]))
 	{
-		lerp = (cl.time - cl.movement_time[2]) / (cl.movement_time[0] - cl.movement_time[1]);
-		lerp = bound(0, lerp, 1);
-		if (!interpolate)
-			lerp = 1;
-		VectorLerp(cl.movement_oldorigin, lerp, cl.movement_origin, origin);
+		VectorCopy(cl.movement_origin, origin);
 		VectorSet(angles, 0, cl.viewangles[1], 0);
 	}
 	else if (interpolate && e->persistent.lerpdeltatime > 0 && (lerp = (cl.time - e->persistent.lerpstarttime) / e->persistent.lerpdeltatime) < 1)
@@ -2216,7 +2212,7 @@ void CL_Init (void)
 	r_refdef.scene.maxentities = MAX_EDICTS + 256 + 512;
 	r_refdef.scene.entities = (entity_render_t **)Mem_Alloc(cls.permanentmempool, sizeof(entity_render_t *) * r_refdef.scene.maxentities);
 
-	r_refdef.scene.maxtempentities = 512;
+	r_refdef.scene.maxtempentities = 4096; // FIXME: make this grow
 	r_refdef.scene.tempentities = (entity_render_t *)Mem_Alloc(cls.permanentmempool, sizeof(entity_render_t) * r_refdef.scene.maxtempentities);
 
 	CL_InitInput ();
