@@ -1722,6 +1722,7 @@ static void R_Envmap_f (void)
 	r_refdef.view.height = size;
 	r_refdef.view.depth = 1;
 	r_refdef.view.useperspective = true;
+	r_refdef.view.isoverlay = false;
 
 	r_refdef.view.frustum_x = tan(90 * M_PI / 360.0);
 	r_refdef.view.frustum_y = tan(90 * M_PI / 360.0);
@@ -2080,14 +2081,14 @@ void SCR_UpdateLoadingScreen (qboolean clear)
 	if (vid.stereobuffer)
 	{
 		qglDrawBuffer(GL_FRONT_LEFT);
-		R_Mesh_Draw(0, 4, 2, polygonelements, 0, 0);
+		R_Mesh_Draw(0, 4, 0, 2, NULL, polygonelements, 0, 0);
 		qglDrawBuffer(GL_FRONT_RIGHT);
-		R_Mesh_Draw(0, 4, 2, polygonelements, 0, 0);
+		R_Mesh_Draw(0, 4, 0, 2, NULL, polygonelements, 0, 0);
 	}
 	else
 	{
 		qglDrawBuffer(GL_FRONT);
-		R_Mesh_Draw(0, 4, 2, polygonelements, 0, 0);
+		R_Mesh_Draw(0, 4, 0, 2, NULL, polygonelements, 0, 0);
 	}
 	R_Mesh_Finish();
 	// refresh
@@ -2110,15 +2111,18 @@ void CL_UpdateScreen(void)
 	double rendertime1;
 	float conwidth, conheight;
 
-	// play a bit with the palette
-	palette_rgb_pantscolormap[15][0] = 128 + 127 * sin(cl.time / exp(1) + 0*M_PI/3);
-	palette_rgb_pantscolormap[15][1] = 128 + 127 * sin(cl.time / exp(1) + 2*M_PI/3);
-	palette_rgb_pantscolormap[15][2] = 128 + 127 * sin(cl.time / exp(1) + 4*M_PI/3);
-	palette_rgb_shirtcolormap[15][0] = 128 + 127 * sin(cl.time /  M_PI  + 5*M_PI/3);
-	palette_rgb_shirtcolormap[15][1] = 128 + 127 * sin(cl.time /  M_PI  + 3*M_PI/3);
-	palette_rgb_shirtcolormap[15][2] = 128 + 127 * sin(cl.time /  M_PI  + 1*M_PI/3);
-	memcpy(palette_rgb_pantsscoreboard[15], palette_rgb_pantscolormap[15], sizeof(*palette_rgb_pantscolormap));
-	memcpy(palette_rgb_shirtscoreboard[15], palette_rgb_shirtcolormap[15], sizeof(*palette_rgb_shirtcolormap));
+	if(gamemode == GAME_NEXUIZ)
+	{
+		// play a bit with the palette (experimental)
+		palette_rgb_pantscolormap[15][0] = 128 + 127 * sin(cl.time / exp(1) + 0*M_PI/3);
+		palette_rgb_pantscolormap[15][1] = 128 + 127 * sin(cl.time / exp(1) + 2*M_PI/3);
+		palette_rgb_pantscolormap[15][2] = 128 + 127 * sin(cl.time / exp(1) + 4*M_PI/3);
+		palette_rgb_shirtcolormap[15][0] = 128 + 127 * sin(cl.time /  M_PI  + 5*M_PI/3);
+		palette_rgb_shirtcolormap[15][1] = 128 + 127 * sin(cl.time /  M_PI  + 3*M_PI/3);
+		palette_rgb_shirtcolormap[15][2] = 128 + 127 * sin(cl.time /  M_PI  + 1*M_PI/3);
+		memcpy(palette_rgb_pantsscoreboard[15], palette_rgb_pantscolormap[15], sizeof(*palette_rgb_pantscolormap));
+		memcpy(palette_rgb_shirtscoreboard[15], palette_rgb_shirtcolormap[15], sizeof(*palette_rgb_shirtcolormap));
+	}
 
 	if (vid_hidden || !scr_refresh.integer)
 		return;
@@ -2185,6 +2189,7 @@ void CL_UpdateScreen(void)
 	qglClearColor(0,0,0,0);CHECKGLERROR
 	R_ClearScreen(false);
 	r_refdef.view.clear = false;
+	r_refdef.view.isoverlay = false;
 	r_refdef.view.quality = bound(cl_minfps_qualitymin.value, pow(cl_updatescreen_quality, cl_minfps_qualitypower.value) * cl_minfps_qualityscale.value, cl_minfps_qualitymax.value);
 
 	if(scr_stipple.integer)
