@@ -19,14 +19,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // common.c -- misc functions used in client and server
 
+#include "quakedef.h"
+
 #include <stdlib.h>
 #include <fcntl.h>
 #ifndef WIN32
 #include <unistd.h>
 #endif
 
-#include "quakedef.h"
-
 cvar_t registered = {0, "registered","0", "indicates if this is running registered quake (whether gfx/pop.lmp was found)"};
 cvar_t cmdline = {0, "cmdline","0", "contains commandline the engine was launched with"};
 
@@ -173,6 +173,14 @@ unsigned short CRC_Block(const unsigned char *data, size_t size)
 	return crc ^ CRC_XOR_VALUE;
 }
 
+unsigned short CRC_Block_CaseInsensitive(const unsigned char *data, size_t size)
+{
+	unsigned short crc = CRC_INIT_VALUE;
+	while (size--)
+		crc = (crc << 8) ^ crctable[(crc >> 8) ^ (tolower(*data++))];
+	return crc ^ CRC_XOR_VALUE;
+}
+
 // QuakeWorld
 static unsigned char chktbl[1024 + 4] =
 {
@@ -334,7 +342,7 @@ void MSG_WriteCoord32f (sizebuf_t *sb, float f)
 
 void MSG_WriteCoord (sizebuf_t *sb, float f, protocolversion_t protocol)
 {
-	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_QUAKEWORLD)
+	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_NEHAHRABJP || protocol == PROTOCOL_NEHAHRABJP2 || protocol == PROTOCOL_NEHAHRABJP3 || protocol == PROTOCOL_QUAKEWORLD)
 		MSG_WriteCoord13i (sb, f);
 	else if (protocol == PROTOCOL_DARKPLACES1)
 		MSG_WriteCoord32f (sb, f);
@@ -375,7 +383,7 @@ void MSG_WriteAngle32f (sizebuf_t *sb, float f)
 
 void MSG_WriteAngle (sizebuf_t *sb, float f, protocolversion_t protocol)
 {
-	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_DARKPLACES1 || protocol == PROTOCOL_DARKPLACES2 || protocol == PROTOCOL_DARKPLACES3 || protocol == PROTOCOL_DARKPLACES4 || protocol == PROTOCOL_QUAKEWORLD)
+	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_NEHAHRABJP || protocol == PROTOCOL_NEHAHRABJP2 || protocol == PROTOCOL_NEHAHRABJP3 || protocol == PROTOCOL_DARKPLACES1 || protocol == PROTOCOL_DARKPLACES2 || protocol == PROTOCOL_DARKPLACES3 || protocol == PROTOCOL_DARKPLACES4 || protocol == PROTOCOL_QUAKEWORLD)
 		MSG_WriteAngle8i (sb, f);
 	else
 		MSG_WriteAngle16i (sb, f);
@@ -506,7 +514,7 @@ float MSG_ReadCoord32f (void)
 
 float MSG_ReadCoord (protocolversion_t protocol)
 {
-	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_QUAKEWORLD)
+	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_NEHAHRABJP || protocol == PROTOCOL_NEHAHRABJP2 || protocol == PROTOCOL_NEHAHRABJP3 || protocol == PROTOCOL_QUAKEWORLD)
 		return MSG_ReadCoord13i();
 	else if (protocol == PROTOCOL_DARKPLACES1)
 		return MSG_ReadCoord32f();
@@ -541,7 +549,7 @@ float MSG_ReadAngle32f (void)
 
 float MSG_ReadAngle (protocolversion_t protocol)
 {
-	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_DARKPLACES1 || protocol == PROTOCOL_DARKPLACES2 || protocol == PROTOCOL_DARKPLACES3 || protocol == PROTOCOL_DARKPLACES4 || protocol == PROTOCOL_QUAKEWORLD)
+	if (protocol == PROTOCOL_QUAKE || protocol == PROTOCOL_QUAKEDP || protocol == PROTOCOL_NEHAHRAMOVIE || protocol == PROTOCOL_NEHAHRABJP || protocol == PROTOCOL_NEHAHRABJP2 || protocol == PROTOCOL_NEHAHRABJP3 || protocol == PROTOCOL_DARKPLACES1 || protocol == PROTOCOL_DARKPLACES2 || protocol == PROTOCOL_DARKPLACES3 || protocol == PROTOCOL_DARKPLACES4 || protocol == PROTOCOL_QUAKEWORLD)
 		return MSG_ReadAngle8i ();
 	else
 		return MSG_ReadAngle16i ();
@@ -666,18 +674,108 @@ void SZ_HexDumpToConsole(const sizebuf_t *buf)
 ==============
 COM_Wordwrap
 
-Word wraps a string. The charWidth function is guaranteed to be called exactly
-once for each character in the string, so it may be stateful and e.g. implement
-kerning. At the beginning of the string, it will be called for the char 0 to
-initialize a clean state; the return value in this case however gets ignored.
+Word wraps a string. The wordWidth function is guaranteed to be called exactly
+once for each word in the string, so it may be stateful, no idea what that
+would be good for any more. At the beginning of the string, it will be called
+for the char 0 to initialize a clean state, and then once with the string " "
+(a space) so the routine knows how long a space is.
 
 Wrapped lines get the isContinuation flag set and are continuationWidth less wide.
 
 The sum of the return values of the processLine function will be returned.
 ==============
 */
-int COM_Wordwrap(const char *string, size_t length, float continuationWidth, float maxWidth, COM_CharWidthFunc_t charWidth, void *passthroughCW, COM_LineProcessorFunc processLine, void *passthroughPL)
+int COM_Wordwrap(const char *string, size_t length, float continuationWidth, float maxWidth, COM_WordWidthFunc_t wordWidth, void *passthroughCW, COM_LineProcessorFunc processLine, void *passthroughPL)
 {
+	// Logic is as follows:
+	//
+	// For each word or whitespace:
+	//   Newline found? Output current line, advance to next line. This is not a continuation. Continue.
+	//   Space found? Always add it to the current line, no matter if it fits.
+	//   Word found? Check if current line + current word fits.
+	//     If it fits, append it. Continue.
+	//     If it doesn't fit, output current line, advance to next line. Append the word. This is a continuation. Continue.
+
+	qboolean isContinuation = false;
+	float spaceWidth;
+	const char *startOfLine = string;
+	const char *cursor = string;
+	const char *end = string + length;
+	float spaceUsedInLine = 0;
+	float spaceUsedForWord;
+	int result = 0;
+	size_t wordLen;
+	size_t dummy;
+
+	dummy = 0;
+	wordWidth(passthroughCW, NULL, &dummy, -1);
+	dummy = 1;
+	spaceWidth = wordWidth(passthroughCW, " ", &dummy, -1);
+
+	for(;;)
+	{
+		char ch = (cursor < end) ? *cursor : 0;
+		switch(ch)
+		{
+			case 0: // end of string
+				result += processLine(passthroughPL, startOfLine, cursor - startOfLine, spaceUsedInLine, isContinuation);
+				isContinuation = false;
+				goto out;
+				break;
+			case '\n': // end of line
+				result += processLine(passthroughPL, startOfLine, cursor - startOfLine, spaceUsedInLine, isContinuation);
+				isContinuation = false;
+				++cursor;
+				startOfLine = cursor;
+				break;
+			case ' ': // space
+				++cursor;
+				spaceUsedInLine += spaceWidth;
+				break;
+			default: // word
+				wordLen = 1;
+				while(cursor + wordLen < end)
+				{
+					switch(cursor[wordLen])
+					{
+						case 0:
+						case '\n':
+						case ' ':
+							goto out_inner;
+						default:
+							++wordLen;
+							break;
+					}
+				}
+				out_inner:
+				spaceUsedForWord = wordWidth(passthroughCW, cursor, &wordLen, maxWidth - continuationWidth); // this may have reduced wordLen when it won't fit - but this is GOOD. TODO fix words that do fit in a non-continuation line
+				if(wordLen < 1)
+				{
+					wordLen = 1;
+					spaceUsedForWord = maxWidth + 1; // too high, forces it in a line of itself
+				}
+				if(spaceUsedInLine + spaceUsedForWord <= maxWidth || cursor == startOfLine)
+				{
+					// we can simply append it
+					cursor += wordLen;
+					spaceUsedInLine += spaceUsedForWord;
+				}
+				else
+				{
+					// output current line
+					result += processLine(passthroughPL, startOfLine, cursor - startOfLine, spaceUsedInLine, isContinuation);
+					isContinuation = true;
+					startOfLine = cursor;
+					cursor += wordLen;
+					spaceUsedInLine = continuationWidth + spaceUsedForWord;
+				}
+		}
+	}
+	out:
+
+	return result;
+
+/*
 	qboolean isContinuation = false;
 	float currentWordSpace = 0;
 	const char *currentWord = 0;
@@ -690,8 +788,8 @@ int COM_Wordwrap(const char *string, size_t length, float continuationWidth, flo
 	const char *p;
 
 	int result = 0;
-	charWidth(passthroughCW, 0);
-	minReserve = charWidth(passthroughCW, 'm') + charWidth(passthroughCW, ' '); // FIXME assuming that is 'm' fits, anything does
+	minReserve = charWidth(passthroughCW, 0);
+	minReserve += charWidth(passthroughCW, ' ');
 
 	if(maxWidth < continuationWidth + minReserve)
 		maxWidth = continuationWidth + minReserve;
@@ -867,6 +965,7 @@ int COM_Wordwrap(const char *string, size_t length, float continuationWidth, flo
 	}
 
 	return result;
+*/
 }
 
 /*
@@ -876,7 +975,7 @@ COM_ParseToken_Simple
 Parse a token out of a string
 ==============
 */
-int COM_ParseToken_Simple(const char **datapointer, int returnnewline)
+int COM_ParseToken_Simple(const char **datapointer, qboolean returnnewline, qboolean parsebackslash)
 {
 	int len;
 	int c;
@@ -933,16 +1032,10 @@ skipwhite:
 	else if (*data == '\"')
 	{
 		// quoted string
-		for (data++;*data != '\"';data++)
+		for (data++;*data && *data != '\"';data++)
 		{
-			if (!*data || len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
 			c = *data;
-			if (*data == '\\')
+			if (*data == '\\' && parsebackslash)
 			{
 				data++;
 				c = *data;
@@ -951,10 +1044,13 @@ skipwhite:
 				else if (c == 't')
 					c = '\t';
 			}
-			com_token[len++] = c;
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = c;
 		}
 		com_token[len] = 0;
-		*datapointer = data+1;
+		if (*data == '\"')
+			data++;
+		*datapointer = data;
 		return true;
 	}
 	else if (*data == '\r')
@@ -977,15 +1073,8 @@ skipwhite:
 	{
 		// regular word
 		for (;*data > ' ';data++)
-		{
-			if (len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
-			com_token[len++] = *data;
-		}
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = *data;
 		com_token[len] = 0;
 		*datapointer = data;
 		return true;
@@ -999,7 +1088,7 @@ COM_ParseToken_QuakeC
 Parse a token out of a string
 ==============
 */
-int COM_ParseToken_QuakeC(const char **datapointer, int returnnewline)
+int COM_ParseToken_QuakeC(const char **datapointer, qboolean returnnewline)
 {
 	int len;
 	int c;
@@ -1057,14 +1146,8 @@ skipwhite:
 	{
 		// quoted string
 		char quote = *data;
-		for (data++;*data != quote;data++)
+		for (data++;*data && *data != quote;data++)
 		{
-			if (!*data || len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
 			c = *data;
 			if (*data == '\\')
 			{
@@ -1075,10 +1158,13 @@ skipwhite:
 				else if (c == 't')
 					c = '\t';
 			}
-			com_token[len++] = c;
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = c;
 		}
 		com_token[len] = 0;
-		*datapointer = data+1;
+		if (*data == quote)
+			data++;
+		*datapointer = data;
 		return true;
 	}
 	else if (*data == '\r')
@@ -1101,15 +1187,8 @@ skipwhite:
 	{
 		// regular word
 		for (;*data > ' ' && *data != '{' && *data != '}' && *data != ')' && *data != '(' && *data != ']' && *data != '[' && *data != ':' && *data != ',' && *data != ';';data++)
-		{
-			if (len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
-			com_token[len++] = *data;
-		}
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = *data;
 		com_token[len] = 0;
 		*datapointer = data;
 		return true;
@@ -1123,7 +1202,7 @@ COM_ParseToken_VM_Tokenize
 Parse a token out of a string
 ==============
 */
-int COM_ParseToken_VM_Tokenize(const char **datapointer, int returnnewline)
+int COM_ParseToken_VM_Tokenize(const char **datapointer, qboolean returnnewline)
 {
 	int len;
 	int c;
@@ -1181,14 +1260,8 @@ skipwhite:
 	{
 		char quote = *data;
 		// quoted string
-		for (data++;*data != quote;data++)
+		for (data++;*data && *data != quote;data++)
 		{
-			if (!*data || len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
 			c = *data;
 			if (*data == '\\')
 			{
@@ -1199,10 +1272,13 @@ skipwhite:
 				else if (c == 't')
 					c = '\t';
 			}
-			com_token[len++] = c;
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = c;
 		}
 		com_token[len] = 0;
-		*datapointer = data+1;
+		if (*data == quote)
+			data++;
+		*datapointer = data;
 		return true;
 	}
 	else if (*data == '\r')
@@ -1225,15 +1301,8 @@ skipwhite:
 	{
 		// regular word
 		for (;*data > ' ' && *data != ',' && *data != ';' && *data != '{' && *data != '}' && *data != ')' && *data != '(' && *data != ']' && *data != '[' && *data != ':' && *data != ',' && *data != ';';data++)
-		{
-			if (len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
-			com_token[len++] = *data;
-		}
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = *data;
 		com_token[len] = 0;
 		*datapointer = data;
 		return true;
@@ -1283,35 +1352,25 @@ skipwhite:
 	else if (*data == '\"')
 	{
 		// quoted string
-		for (data++;*data != '\"';data++)
+		for (data++;*data && *data != '\"';data++)
 		{
-			if (!*data || len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
-			// allow escaped " case
-			if (*data == '\\' && data[1] == '\"')
+			// allow escaped " and \ case
+			if (*data == '\\' && (data[1] == '\"' || data[1] == '\\'))
 				data++;
-			com_token[len++] = *data;
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = *data;
 		}
 		com_token[len] = 0;
-		*datapointer = data+1;
+		if (*data == '\"')
+			data++;
+		*datapointer = data;
 	}
 	else
 	{
 		// regular word
 		for (;*data > ' ';data++)
-		{
-			if (len >= (int)sizeof(com_token) - 1)
-			{
-				com_token[0] = 0;
-				*datapointer = NULL;
-				return false;
-			}
-			com_token[len++] = *data;
-		}
+			if (len < (int)sizeof(com_token) - 1)
+				com_token[len++] = *data;
 		com_token[len] = 0;
 		*datapointer = data;
 	}
@@ -1358,8 +1417,8 @@ typedef struct gamemode_info_s
 	const char* gameuserdirname;
 } gamemode_info_t;
 
-static const gamemode_info_t gamemode_info [] =
-{// prog_name		cmdline			gamename				gamedirname	gamescreenshotname
+static const gamemode_info_t gamemode_info [GAME_COUNT] =
+{// prog_name		cmdline			gamename				basegame	modgame			screenshotprefix	userdir
 
 // GAME_NORMAL
 // COMMANDLINEOPTION: Game: -quake runs the game Quake (default)
@@ -1409,9 +1468,9 @@ static const gamemode_info_t gamemode_info [] =
 // GAME_PRYDON
 // COMMANDLINEOPTION: Game: -prydon runs the topdown point and click action-RPG Prydon Gate
 { "prydon",			"-prydon",		"PrydonGate",			"id1",		"prydon",		"prydon",		"darkplaces" },
-// GAME_NETHERWORLD
-// COMMANDLINEOPTION: Game: -netherworld runs the game Netherworld: Dark Master
-{ "netherworld",	"-netherworld",	"Netherworld: Dark Master",	"id1",		"netherworld", 	"nw",			"darkplaces" },
+// GAME_DELUXEQUAKE
+// COMMANDLINEOPTION: Game: -dq runs the game Deluxe Quake
+{ "dq",	"-dq",	"Deluxe Quake",		"basedq",		"extradq",		"basedq",		"dq" },
 // GAME_THEHUNTED
 // COMMANDLINEOPTION: Game: -thehunted runs the game The Hunted
 { "thehunted",		"-thehunted",	"The Hunted",			"thdata",	NULL, 			"th",			"thehunted" },
@@ -1424,6 +1483,12 @@ static const gamemode_info_t gamemode_info [] =
 // GAME_CONTAGIONTHEORY
 // COMMANDLINEOPTION: Game: -contagiontheory runs the game Contagion Theory
 { "contagiontheory",		"-contagiontheory",	"Contagion Theory",			"ctdata",	NULL, 			"ct",			"contagiontheory" },
+// GAME_EDU2P
+// COMMANDLINEOPTION: Game: -edu2p runs the game Edu2 prototype
+{ "edu2p", "-edu2p", "EDU2 Prototype", "id1", "edu2", "edu2_p", "edu2prototype" },
+// GAME_BLADEMASTER
+// COMMANDLINEOPTION: Game: -blademaster runs the game Prophecy: Return of the BladeMaster
+{ "blademaster", "-blademaster", "Prophecy: Return of the BladeMaster", "basebm", NULL, "blademaster", "blademaster" },
 };
 
 void COM_InitGameType (void)
