@@ -318,6 +318,15 @@ static dllfunction_t zlibfuncs[] =
 // Handle for Zlib DLL
 static dllhandle_t zlib_dll = NULL;
 
+#ifdef WIN32
+static HRESULT (WINAPI *qSHGetFolderPath) (HWND hwndOwner, int nFolder, HANDLE hToken, DWORD dwFlags, LPTSTR pszPath);
+static dllfunction_t shfolderfuncs[] =
+{
+	{"SHGetFolderPathA", (void **) &qSHGetFolderPath},
+	{NULL, NULL}
+};
+static dllhandle_t shfolder_dll = NULL;
+#endif
 
 /*
 ====================
@@ -481,9 +490,16 @@ int PK3_BuildFileList (pack_t *pack, const pk3_endOfCentralDir_t *eocd)
 		// Check encryption, compression, and attributes
 		// 1st uint8  : general purpose bit flag
 		//    Check bits 0 (encryption), 3 (data descriptor after the file), and 5 (compressed patched data (?))
+		//
+		// LordHavoc: bit 3 would be a problem if we were scanning the archive
+		// but is not a problem in the central directory where the values are
+		// always real.
+		//
+		// bit 3 seems to always be set by the standard Mac OSX zip maker
+		//
 		// 2nd uint8 : external file attributes
 		//    Check bits 3 (file is a directory) and 5 (file is a volume (?))
-		if ((ptr[8] & 0x29) == 0 && (ptr[38] & 0x18) == 0)
+		if ((ptr[8] & 0x21) == 0 && (ptr[38] & 0x18) == 0)
 		{
 			// Still enough bytes for the name?
 			if (remaining < namesize || namesize >= (int)sizeof (*pack->files))
@@ -960,12 +976,11 @@ void FS_AddGameDirectory (const char *dir)
 	int i;
 	stringlist_t list;
 	searchpath_t *search;
-	char pakfile[MAX_OSPATH];
 
 	strlcpy (fs_gamedir, dir, sizeof (fs_gamedir));
 
 	stringlistinit(&list);
-	listdirectory(&list, dir);
+	listdirectory(&list, "", dir);
 	stringlistsort(&list);
 
 	// add any PAK package in the directory
@@ -973,8 +988,7 @@ void FS_AddGameDirectory (const char *dir)
 	{
 		if (!strcasecmp(FS_FileExtension(list.strings[i]), "pak"))
 		{
-			dpsnprintf (pakfile, sizeof (pakfile), "%s%s", dir, list.strings[i]);
-			FS_AddPack_Fullpath(pakfile, NULL, false);
+			FS_AddPack_Fullpath(list.strings[i], NULL, false);
 		}
 	}
 
@@ -983,8 +997,7 @@ void FS_AddGameDirectory (const char *dir)
 	{
 		if (!strcasecmp(FS_FileExtension(list.strings[i]), "pk3"))
 		{
-			dpsnprintf (pakfile, sizeof (pakfile), "%s%s", dir, list.strings[i]);
-			FS_AddPack_Fullpath(pakfile, NULL, false);
+			FS_AddPack_Fullpath(list.strings[i], NULL, false);
 		}
 	}
 
@@ -1010,9 +1023,8 @@ void FS_AddGameHierarchy (const char *dir)
 	char userdir[MAX_QPATH];
 #ifdef WIN32
 	TCHAR mydocsdir[MAX_PATH + 1];
-#else
-	const char *homedir;
 #endif
+	const char *homedir;
 
 	// Add the common game directory
 	FS_AddGameDirectory (va("%s%s/", fs_basedir, dir));
@@ -1021,14 +1033,36 @@ void FS_AddGameHierarchy (const char *dir)
 
 	// Add the personal game directory
 #ifdef WIN32
-	if(SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, mydocsdir) == S_OK)
+	if(qSHGetFolderPath && (qSHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, 0, mydocsdir) == S_OK))
+	{
 		dpsnprintf(userdir, sizeof(userdir), "%s/My Games/%s/", mydocsdir, gameuserdirname);
+		Con_DPrintf("Obtained personal directory %s from SHGetFolderPath\n", userdir);
+	}
+	else
+	{
+		// use the environment
+		homedir = getenv ("USERPROFILE");
+		if(homedir)
+		{
+			dpsnprintf(userdir, sizeof(userdir), "%s/My Documents/My Games/%s/", homedir, gameuserdirname);
+			Con_DPrintf("Obtained personal directory %s from environment\n", userdir);
+		}
+		else
+			*userdir = 0; // just to make sure it hasn't been written to by SHGetFolderPath returning failure
+	}
+
+	if(!*userdir)
+		Con_DPrintf("Could not obtain home directory; not supporting -mygames\n");
 #else
 	homedir = getenv ("HOME");
 	if(homedir)
 		dpsnprintf(userdir, sizeof(userdir), "%s/.%s/", homedir, gameuserdirname);
+
+	if(!*userdir)
+		Con_DPrintf("Could not obtain home directory; assuming -nohome\n");
 #endif
 
+
 #ifdef WIN32
 	if(!COM_CheckParm("-mygames"))
 	{
@@ -1321,7 +1355,7 @@ qboolean FS_CheckGameDir(const char *gamedir)
 	qboolean success;
 	stringlist_t list;
 	stringlistinit(&list);
-	listdirectory(&list, va("%s%s/", fs_basedir, gamedir));
+	listdirectory(&list, va("%s%s/", fs_basedir, gamedir), "");
 	success = list.numstrings > 0;
 	stringlistfreecontents(&list);
 	return success;
@@ -1337,6 +1371,16 @@ void FS_Init (void)
 {
 	int i;
 
+#ifdef WIN32
+	const char* dllnames [] =
+	{
+		"shfolder.dll",  // IE 4, or Win NT and higher
+		NULL
+	};
+	Sys_LoadLibrary(dllnames, &shfolder_dll, shfolderfuncs);
+	// don't care for the result; if it fails, %USERPROFILE% will be used instead
+#endif
+
 	fs_mempool = Mem_AllocPool("file management", 0, NULL);
 
 	strlcpy(fs_gamedir, "", sizeof(fs_gamedir));
@@ -1434,6 +1478,10 @@ void FS_Shutdown (void)
 	//  by the OS anyway)
 	FS_ClearSearchPath();
 	Mem_FreePool (&fs_mempool);
+
+#ifdef WIN32
+	Sys_UnloadLibrary (&shfolder_dll);
+#endif
 }
 
 /*
@@ -1811,17 +1859,6 @@ Open a file. The syntax is the same as fopen
 */
 qfile_t* FS_Open (const char* filepath, const char* mode, qboolean quiet, qboolean nonblocking)
 {
-#ifdef FS_FIX_PATHS
-	char fixedFileName[MAX_QPATH];
-	char *d;
-	strlcpy( fixedFileName, filepath, MAX_QPATH );
-	// try to fix common mistakes (\ instead of /)
-	for( d = fixedFileName ; *d ; d++ )
-		if( *d == '\\' )
-			*d = '/';
-	filepath = fixedFileName;
-#endif
-
 	if (FS_CheckNastyPath(filepath, false))
 	{
 		Con_Printf("FS_Open(\"%s\", \"%s\", %s): nasty filename rejected\n", filepath, mode, quiet ? "true" : "false");
@@ -2529,7 +2566,6 @@ fssearch_t *FS_Search(const char *pattern, int caseinsensitive, int quiet)
 	stringlist_t dirlist;
 	const char *slash, *backslash, *colon, *separator;
 	char *basepath;
-	char netpath[MAX_OSPATH];
 	char temp[MAX_OSPATH];
 
 	for (i = 0;pattern[i] == '.' || pattern[i] == ':' || pattern[i] == '/' || pattern[i] == '\\';i++)
@@ -2598,13 +2634,80 @@ fssearch_t *FS_Search(const char *pattern, int caseinsensitive, int quiet)
 		}
 		else
 		{
-			// get a directory listing and look at each name
-			dpsnprintf(netpath, sizeof (netpath), "%s%s", searchpath->filename, basepath);
-			stringlistinit(&dirlist);
-			listdirectory(&dirlist, netpath);
-			for (dirlistindex = 0;dirlistindex < dirlist.numstrings;dirlistindex++)
-  			{
-				dpsnprintf(temp, sizeof(temp), "%s%s", basepath, dirlist.strings[dirlistindex]);
+			stringlist_t matchedSet, foundSet;
+			const char *start = pattern;
+
+			stringlistinit(&matchedSet);
+			stringlistinit(&foundSet);
+			// add a first entry to the set
+			stringlistappend(&matchedSet, "");
+			// iterate through pattern's path
+			while (*start)
+			{
+				const char *asterisk, *wildcard, *nextseparator, *prevseparator;
+				char subpath[MAX_OSPATH];
+				char subpattern[MAX_OSPATH];
+
+				// find the next wildcard
+				wildcard = strchr(start, '?');
+				asterisk = strchr(start, '*');
+				if (asterisk && (!wildcard || asterisk < wildcard))
+				{
+					wildcard = asterisk;
+				}
+
+				if (wildcard)
+				{
+					nextseparator = strchr( wildcard, '/' );
+				}
+				else
+				{
+					nextseparator = NULL;
+				}
+
+				if( !nextseparator ) {
+					nextseparator = start + strlen( start );
+				}
+
+				// prevseparator points past the '/' right before the wildcard and nextseparator at the one following it (or at the end of the string)
+				// copy everything up except nextseperator
+				strlcpy(subpattern, pattern, min(sizeof(subpattern), (size_t) (nextseparator - pattern + 1)));
+				// find the last '/' before the wildcard
+				prevseparator = strrchr( subpattern, '/' );
+				if (!prevseparator)
+					prevseparator = subpattern;
+				else
+					prevseparator++;
+				// copy everything from start to the previous including the '/' (before the wildcard)
+				// everything up to start is already included in the path of matchedSet's entries
+				strlcpy(subpath, start, min(sizeof(subpath), (size_t) ((prevseparator - subpattern) - (start - pattern) + 1)));
+
+				// for each entry in matchedSet try to open the subdirectories specified in subpath
+				for( dirlistindex = 0 ; dirlistindex < matchedSet.numstrings ; dirlistindex++ ) {
+					strlcpy( temp, matchedSet.strings[ dirlistindex ], sizeof(temp) );
+					strlcat( temp, subpath, sizeof(temp) );
+					listdirectory( &foundSet, searchpath->filename, temp );
+				}
+				if( dirlistindex == 0 ) {
+					break;
+				}
+				// reset the current result set
+				stringlistfreecontents( &matchedSet );
+				// match against the pattern
+				for( dirlistindex = 0 ; dirlistindex < foundSet.numstrings ; dirlistindex++ ) {
+					const char *direntry = foundSet.strings[ dirlistindex ];
+					if (matchpattern(direntry, subpattern, true)) {
+						stringlistappend( &matchedSet, direntry );
+					}
+				}
+				stringlistfreecontents( &foundSet );
+
+				start = nextseparator;
+			}
+
+			for (dirlistindex = 0;dirlistindex < matchedSet.numstrings;dirlistindex++)
+			{
+				const char *temp = matchedSet.strings[dirlistindex];
 				if (matchpattern(temp, (char *)pattern, true))
 				{
 					for (resultlistindex = 0;resultlistindex < resultlist.numstrings;resultlistindex++)
@@ -2618,7 +2721,7 @@ fssearch_t *FS_Search(const char *pattern, int caseinsensitive, int quiet)
 					}
 				}
 			}
-			stringlistfreecontents(&dirlist);
+			stringlistfreecontents( &matchedSet );
 		}
 	}
 
