@@ -28,7 +28,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 cvar_t r_ambient = {0, "r_ambient", "0", "brightens map, value is 0-128"};
 cvar_t r_lockpvs = {0, "r_lockpvs", "0", "disables pvs switching, allows you to walk around and inspect what is visible from a given location in the map (anything not visible from your current location will not be drawn)"};
 cvar_t r_lockvisibility = {0, "r_lockvisibility", "0", "disables visibility updates, allows you to walk around and inspect what is visible from a given viewpoint in the map (anything offscreen at the moment this is enabled will not be drawn)"};
-cvar_t r_useportalculling = {0, "r_useportalculling", "1", "use advanced portal culling visibility method to improve performance over just Potentially Visible Set, provides an even more significant speed improvement in unvised maps"};
+cvar_t r_useportalculling = {0, "r_useportalculling", "1", "improve framerate with r_novis 1 by using portal culling - still not as good as compiled visibility data in the map, but it helps (a value of 2 forces use of this even with vis data, which improves framerates in maps without too much complexity, but hurts in extremely complex maps, which is why 2 is not the default mode)"};
 cvar_t r_q3bsp_renderskydepth = {0, "r_q3bsp_renderskydepth", "0", "draws sky depth masking in q3 maps (as in q1 maps), this means for example that sky polygons can hide other things"};
 
 /*
@@ -43,7 +43,7 @@ void R_BuildLightMap (const entity_render_t *ent, msurface_t *surface)
 	int smax, tmax, i, size, size3, maps, l;
 	int *bl, scale;
 	unsigned char *lightmap, *out, *stain;
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	int *intblocklights;
 	unsigned char *templight;
 
@@ -99,12 +99,25 @@ void R_BuildLightMap (const entity_render_t *ent, msurface_t *surface)
 	// scaling, and remaps the 0-65536 (2x overbright) to 0-256, it will
 	// be doubled during rendering to achieve 2x overbright
 	// (0 = 0.0, 128 = 1.0, 256 = 2.0)
-	for (i = 0;i < size;i++, bl += 3, stain += 3, out += 4)
+	if (stain)
 	{
-		l = (bl[0] * stain[0]) >> 16;out[2] = min(l, 255);
-		l = (bl[1] * stain[1]) >> 16;out[1] = min(l, 255);
-		l = (bl[2] * stain[2]) >> 16;out[0] = min(l, 255);
-		out[3] = 255;
+		for (i = 0;i < size;i++, bl += 3, stain += 3, out += 4)
+		{
+			l = (bl[0] * stain[0]) >> 16;out[2] = min(l, 255);
+			l = (bl[1] * stain[1]) >> 16;out[1] = min(l, 255);
+			l = (bl[2] * stain[2]) >> 16;out[0] = min(l, 255);
+			out[3] = 255;
+		}
+	}
+	else
+	{
+		for (i = 0;i < size;i++, bl += 3, out += 4)
+		{
+			l = bl[0] >> 8;out[2] = min(l, 255);
+			l = bl[1] >> 8;out[1] = min(l, 255);
+			l = bl[2] >> 8;out[0] = min(l, 255);
+			out[3] = 255;
+		}
 	}
 
 	R_UpdateTexture(surface->lightmaptexture, templight, surface->lightmapinfo->lightmaporigin[0], surface->lightmapinfo->lightmaporigin[1], smax, tmax);
@@ -149,7 +162,7 @@ void R_BuildLightMap (const entity_render_t *ent, msurface_t *surface)
 	}
 }
 
-void R_StainNode (mnode_t *node, model_t *model, const vec3_t origin, float radius, const float fcolor[8])
+void R_StainNode (mnode_t *node, dp_model_t *model, const vec3_t origin, float radius, const float fcolor[8])
 {
 	float ndist, a, ratio, maxdist, maxdist2, maxdist3, invradius, sdtable[256], td, dist2;
 	msurface_t *surface, *endsurface;
@@ -276,7 +289,7 @@ void R_Stain (const vec3_t origin, float radius, int cr1, int cg1, int cb1, int
 	int n;
 	float fcolor[8];
 	entity_render_t *ent;
-	model_t *model;
+	dp_model_t *model;
 	vec3_t org;
 	if (r_refdef.scene.worldmodel == NULL || !r_refdef.scene.worldmodel->brush.data_nodes || !r_refdef.scene.worldmodel->brushq1.lightdata)
 		return;
@@ -348,7 +361,7 @@ static void R_DrawPortal_Callback(const entity_render_t *ent, const rtlight_t *r
 			 0.125f);
 	for (i = 0, v = vertex3f;i < numpoints;i++, v += 3)
 		VectorCopy(portal->points[i].position, v);
-	R_Mesh_Draw(0, numpoints, numpoints - 2, polygonelements, 0, 0);
+	R_Mesh_Draw(0, numpoints, 0, numpoints - 2, NULL, polygonelements, 0, 0);
 }
 
 // LordHavoc: this is just a nice debugging tool, very slow
@@ -357,7 +370,7 @@ void R_DrawPortals(void)
 	int i, leafnum;
 	mportal_t *portal;
 	float center[3], f;
-	model_t *model = r_refdef.scene.worldmodel;
+	dp_model_t *model = r_refdef.scene.worldmodel;
 	if (model == NULL)
 		return;
 	for (leafnum = 0;leafnum < r_refdef.scene.worldmodel->brush.num_leafs;leafnum++)
@@ -387,7 +400,7 @@ void R_View_WorldVisibility(qboolean forcenovis)
 	int i, j, *mark;
 	mleaf_t *leaf;
 	mleaf_t *viewleaf;
-	model_t *model = r_refdef.scene.worldmodel;
+	dp_model_t *model = r_refdef.scene.worldmodel;
 
 	if (!model)
 		return;
@@ -450,9 +463,9 @@ void R_View_WorldVisibility(qboolean forcenovis)
 				}
 			}
 		}
-		// if the user prefers to disable portal culling (testing?), simply
-		// use all on-screen leafs that are in the pvs.
-		else if (!r_useportalculling.integer)
+		// just check if each leaf in the PVS is on screen
+		// (unless portal culling is enabled)
+		else if (!model->brush.data_portals || r_useportalculling.integer < 1 || (r_useportalculling.integer < 2 && !r_novis.integer))
 		{
 			// pvs method:
 			// simply check if each leaf is in the Potentially Visible Set,
@@ -471,7 +484,7 @@ void R_View_WorldVisibility(qboolean forcenovis)
 				}
 			}
 		}
-		// otherwise use a recursive portal flow, culling each portal to
+		// if desired use a recursive portal flow, culling each portal to
 		// frustum and checking if the leaf the portal leads to is in the pvs
 		else
 		{
@@ -532,7 +545,7 @@ void R_Q1BSP_DrawSky(entity_render_t *ent)
 
 void R_Q1BSP_DrawAddWaterPlanes(entity_render_t *ent)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	if (model == NULL)
 		return;
 	if (ent == r_refdef.scene.worldentity)
@@ -543,7 +556,7 @@ void R_Q1BSP_DrawAddWaterPlanes(entity_render_t *ent)
 
 void R_Q1BSP_Draw(entity_render_t *ent)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	if (model == NULL)
 		return;
 	if (ent == r_refdef.scene.worldentity)
@@ -554,7 +567,7 @@ void R_Q1BSP_Draw(entity_render_t *ent)
 
 void R_Q1BSP_DrawDepth(entity_render_t *ent)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	if (model == NULL)
 		return;
 	GL_ColorMask(0,0,0,0);
@@ -585,7 +598,7 @@ void R_Q1BSP_DrawDebug(entity_render_t *ent)
 
 typedef struct r_q1bsp_getlightinfo_s
 {
-	model_t *model;
+	dp_model_t *model;
 	vec3_t relativelightorigin;
 	float lightradius;
 	int *outleaflist;
@@ -912,12 +925,12 @@ void R_Q1BSP_GetLightInfo(entity_render_t *ent, vec3_t relativelightorigin, floa
 		info.pvs = NULL;
 	R_UpdateAllTextureInfo(ent);
 
-	if (r_shadow_frontsidecasting.integer && r_shadow_compilingrtlight && r_shadow_realtime_world_compileportalculling.integer)
+	if (r_shadow_frontsidecasting.integer && r_shadow_compilingrtlight && r_shadow_realtime_world_compileportalculling.integer && info.model->brush.data_portals)
 	{
 		// use portal recursion for exact light volume culling, and exact surface checking
 		Portal_Visibility(info.model, info.relativelightorigin, info.outleaflist, info.outleafpvs, &info.outnumleafs, info.outsurfacelist, info.outsurfacepvs, &info.outnumsurfaces, NULL, 0, true, info.lightmins, info.lightmaxs, info.outmins, info.outmaxs, info.outshadowtrispvs, info.outlighttrispvs);
 	}
-	else if (r_shadow_frontsidecasting.integer && r_shadow_realtime_dlight_portalculling.integer)
+	else if (r_shadow_frontsidecasting.integer && r_shadow_realtime_dlight_portalculling.integer && info.model->brush.data_portals)
 	{
 		// use portal recursion for exact light volume culling, but not the expensive exact surface checking
 		Portal_Visibility(info.model, info.relativelightorigin, info.outleaflist, info.outleafpvs, &info.outnumleafs, info.outsurfacelist, info.outsurfacepvs, &info.outnumsurfaces, NULL, 0, r_shadow_realtime_dlight_portalculling.integer >= 2, info.lightmins, info.lightmaxs, info.outmins, info.outmaxs, info.outshadowtrispvs, info.outlighttrispvs);
@@ -945,7 +958,7 @@ void R_Q1BSP_GetLightInfo(entity_render_t *ent, vec3_t relativelightorigin, floa
 
 void R_Q1BSP_CompileShadowVolume(entity_render_t *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int numsurfaces, const int *surfacelist)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	msurface_t *surface;
 	int surfacelistindex;
 	float projectdistance = relativelightdirection ? lightradius : lightradius + model->radius*2 + r_shadow_projectdistance.value;
@@ -966,7 +979,7 @@ extern cvar_t r_polygonoffset_submodel_factor;
 extern cvar_t r_polygonoffset_submodel_offset;
 void R_Q1BSP_DrawShadowVolume(entity_render_t *ent, vec3_t relativelightorigin, vec3_t relativelightdirection, float lightradius, int modelnumsurfaces, const int *modelsurfacelist, const vec3_t lightmins, const vec3_t lightmaxs)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	msurface_t *surface;
 	int modelsurfacelistindex;
 	float projectdistance = relativelightdirection ? lightradius : lightradius + model->radius*2 + r_shadow_projectdistance.value;
@@ -1034,7 +1047,7 @@ static void R_Q1BSP_DrawLight_TransparentCallback(const entity_render_t *ent, co
 			if (t != surface->texture)
 				break;
 			RSurf_PrepareVerticesForBatch(true, true, 1, &surface);
-			R_Shadow_RenderLighting(surface->num_firstvertex, surface->num_vertices, surface->num_triangles, ent->model->surfmesh.data_element3i + surface->num_firsttriangle * 3, ent->model->surfmesh.ebo, (sizeof(int[3]) * surface->num_firsttriangle));
+			R_Shadow_RenderLighting(surface->num_firstvertex, surface->num_vertices, surface->num_firsttriangle, surface->num_triangles, ent->model->surfmesh.data_element3i, ent->model->surfmesh.data_element3s, ent->model->surfmesh.ebo3i, ent->model->surfmesh.ebo3s);
 		}
 	}
 	R_Shadow_RenderMode_End();
@@ -1044,7 +1057,7 @@ static void R_Q1BSP_DrawLight_TransparentCallback(const entity_render_t *ent, co
 
 void R_Q1BSP_DrawLight(entity_render_t *ent, int numsurfaces, const int *surfacelist, const unsigned char *trispvs)
 {
-	model_t *model = ent->model;
+	dp_model_t *model = ent->model;
 	msurface_t *surface;
 	int i, k, kend, l, m, mend, endsurface, batchnumsurfaces, batchnumtriangles, batchfirstvertex, batchlastvertex, batchfirsttriangle;
 	qboolean usebufferobject, culltriangles;
@@ -1134,9 +1147,9 @@ void R_Q1BSP_DrawLight(entity_render_t *ent, int numsurfaces, const int *surface
 						Mod_VertexRangeFromElements(batchnumtriangles*3, batchelements, &batchfirstvertex, &batchlastvertex);
 						// use the element buffer if all triangles are consecutive
 						if (m == batchfirsttriangle + batchnumtriangles)
-							R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchnumtriangles, batchelements, ent->model->surfmesh.ebo, sizeof(int[3]) * batchfirsttriangle);
+							R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchfirsttriangle, batchnumtriangles, ent->model->surfmesh.data_element3i, ent->model->surfmesh.data_element3s, ent->model->surfmesh.ebo3i, ent->model->surfmesh.ebo3s);
 						else
-							R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchnumtriangles, batchelements, 0, 0);
+							R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, 0, batchnumtriangles, batchelements, NULL, 0, 0);
 						usebufferobject = true;
 						batchnumtriangles = 0;
 						batchfirsttriangle = m;
@@ -1153,9 +1166,9 @@ void R_Q1BSP_DrawLight(entity_render_t *ent, int numsurfaces, const int *surface
 				Mod_VertexRangeFromElements(batchnumtriangles*3, batchelements, &batchfirstvertex, &batchlastvertex);
 				// use the element buffer if all triangles are consecutive
 				if (m == batchfirsttriangle + batchnumtriangles)
-					R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchnumtriangles, batchelements, ent->model->surfmesh.ebo, sizeof(int[3]) * batchfirsttriangle);
+					R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchfirsttriangle, batchnumtriangles, ent->model->surfmesh.data_element3i, ent->model->surfmesh.data_element3s, ent->model->surfmesh.ebo3i, ent->model->surfmesh.ebo3s);
 				else
-					R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, batchnumtriangles, batchelements, 0, 0);
+					R_Shadow_RenderLighting(batchfirstvertex, batchlastvertex + 1 - batchfirstvertex, 0, batchnumtriangles, batchelements, NULL, 0, 0);
 			}
 		}
 	}
@@ -1164,7 +1177,7 @@ void R_Q1BSP_DrawLight(entity_render_t *ent, int numsurfaces, const int *surface
 //Made by [515]
 void R_ReplaceWorldTexture (void)
 {
-	model_t		*m;
+	dp_model_t		*m;
 	texture_t	*t;
 	int			i;
 	const char	*r, *newt;
@@ -1214,7 +1227,7 @@ void R_ReplaceWorldTexture (void)
 //Made by [515]
 void R_ListWorldTextures (void)
 {
-	model_t		*m;
+	dp_model_t		*m;
 	texture_t	*t;
 	int			i;
 	if (!r_refdef.scene.worldmodel)
