@@ -19,13 +19,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 // host.c -- coordinates spawning and killing of local servers
 
-#include <time.h>
 #include "quakedef.h"
+
+#include <time.h>
 #include "libcurl.h"
 #include "cdaudio.h"
+#include "cl_gecko.h"
 #include "cl_video.h"
 #include "progsvm.h"
 #include "csprogs.h"
+#include "sv_demo.h"
 
 /*
 
@@ -56,10 +59,18 @@ jmp_buf host_abortframe;
 cvar_t host_framerate = {0, "host_framerate","0", "locks frame timing to this value in seconds, 0.05 is 20fps for example, note that this can easily run too fast, use cl_maxfps if you want to limit your framerate instead, or sys_ticrate to limit server speed"};
 // shows time used by certain subsystems
 cvar_t host_speeds = {0, "host_speeds","0", "reports how much time is used in server/graphics/sound"};
-// LordHavoc: framerate upper cap
-cvar_t cl_maxfps = {CVAR_SAVE, "cl_maxfps", "1000", "maximum fps cap, if game is running faster than this it will wait before running another frame (useful to make cpu time available to other programs)"};
+cvar_t cl_minfps = {CVAR_SAVE, "cl_minfps", "40", "minimum fps target - while the rendering performance is below this, it will drift toward lower quality"};
+cvar_t cl_minfps_fade = {CVAR_SAVE, "cl_minfps_fade", "0.2", "how fast the quality adapts to varying framerate"};
+cvar_t cl_minfps_qualitymax = {CVAR_SAVE, "cl_minfps_qualitymax", "1", "highest allowed drawdistance multiplier"};
+cvar_t cl_minfps_qualitymin = {CVAR_SAVE, "cl_minfps_qualitymin", "0.25", "lowest allowed drawdistance multiplier"};
+cvar_t cl_minfps_qualitypower = {CVAR_SAVE, "cl_minfps_qualitypower", "4", "raises quality value to a power of itself, higher values make quality drop more sharply in relation to framerate"};
+cvar_t cl_minfps_qualityscale = {CVAR_SAVE, "cl_minfps_qualityscale", "0.5", "multiplier for quality"};
+cvar_t cl_maxfps = {CVAR_SAVE, "cl_maxfps", "1000000", "maximum fps cap, if game is running faster than this it will wait before running another frame (useful to make cpu time available to other programs)"};
+cvar_t cl_maxidlefps = {CVAR_SAVE, "cl_maxidlefps", "20", "maximum fps cap when the game is not the active window (makes cpu time available to other programs"};
 
 cvar_t developer = {0, "developer","0", "prints additional debugging messages and information (recommended for modders and level designers)"};
+cvar_t developer_loadfile = {0, "developer_loadfile","0", "prints name and size of every file loaded via the FS_LoadFile function (which is almost everything)"};
+cvar_t developer_loading = {0, "developer_loading","0", "prints information about files as they are loaded or unloaded successfully"};
 cvar_t developer_entityparsing = {0, "developer_entityparsing", "0", "prints detailed network entities information each time a packet is received"};
 
 cvar_t timestamps = {CVAR_SAVE, "timestamps", "0", "prints timestamps on console messages"};
@@ -193,14 +204,23 @@ void Host_SaveConfig_f(void);
 void Host_LoadConfig_f(void);
 static void Host_InitLocal (void)
 {
-	Cmd_AddCommand("saveconfig", Host_SaveConfig_f, "save settings to config.cfg immediately (also automatic when quitting)");
+	Cmd_AddCommand("saveconfig", Host_SaveConfig_f, "save settings to config.cfg (or a specified filename) immediately (also automatic when quitting)");
 	Cmd_AddCommand("loadconfig", Host_LoadConfig_f, "reset everything and reload configs");
 
 	Cvar_RegisterVariable (&host_framerate);
 	Cvar_RegisterVariable (&host_speeds);
+	Cvar_RegisterVariable (&cl_minfps);
+	Cvar_RegisterVariable (&cl_minfps_fade);
+	Cvar_RegisterVariable (&cl_minfps_qualitymax);
+	Cvar_RegisterVariable (&cl_minfps_qualitymin);
+	Cvar_RegisterVariable (&cl_minfps_qualitypower);
+	Cvar_RegisterVariable (&cl_minfps_qualityscale);
 	Cvar_RegisterVariable (&cl_maxfps);
+	Cvar_RegisterVariable (&cl_maxidlefps);
 
 	Cvar_RegisterVariable (&developer);
+	Cvar_RegisterVariable (&developer_loadfile);
+	Cvar_RegisterVariable (&developer_loading);
 	Cvar_RegisterVariable (&developer_entityparsing);
 
 	Cvar_RegisterVariable (&timestamps);
@@ -215,7 +235,7 @@ Host_SaveConfig_f
 Writes key bindings and archived cvars to config.cfg
 ===============
 */
-void Host_SaveConfig_f(void)
+void Host_SaveConfig_to(const char *file)
 {
 	qfile_t *f;
 
@@ -224,10 +244,10 @@ void Host_SaveConfig_f(void)
 	// LordHavoc: don't save a config if it crashed in startup
 	if (host_framecount >= 3 && cls.state != ca_dedicated && !COM_CheckParm("-benchmark") && !COM_CheckParm("-capturedemo"))
 	{
-		f = FS_Open ("config.cfg", "wb", false, false);
+		f = FS_Open (file, "wb", false, false);
 		if (!f)
 		{
-			Con_Print("Couldn't write config.cfg.\n");
+			Con_Printf("Couldn't write %s.\n", file);
 			return;
 		}
 
@@ -237,7 +257,21 @@ void Host_SaveConfig_f(void)
 		FS_Close (f);
 	}
 }
+void Host_SaveConfig(void)
+{
+	Host_SaveConfig_to("config.cfg");
+}
+void Host_SaveConfig_f(void)
+{
+	const char *file = "config.cfg";
 
+	if(Cmd_Argc() >= 2) {
+		file = Cmd_Argv(1);
+		Con_Printf("Saving to %s\n", file);
+	}
+
+	Host_SaveConfig_to(file);
+}
 
 /*
 ===============
@@ -369,7 +403,9 @@ if (crash = true), don't bother sending signofs
 void SV_DropClient(qboolean crash)
 {
 	int i;
-	Con_Printf("Client \"%s^%i\" dropped\n", host_client->name, STRING_COLOR_DEFAULT);
+	Con_Printf("Client \"%s\" dropped\n", host_client->name);
+
+	SV_StopDemoRecording(host_client);
 
 	// make sure edict is not corrupt (from a level change for example)
 	host_client->edict = PRVM_EDICT_NUM(host_client - svs.clients + 1);
@@ -386,9 +422,9 @@ void SV_DropClient(qboolean crash)
 			buf.data = bufdata;
 			buf.maxsize = sizeof(bufdata);
 			MSG_WriteByte(&buf, svc_disconnect);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
-			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000, false);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000, false);
+			NetConn_SendUnreliableMessage(host_client->netconnection, &buf, sv.protocol, 10000, false);
 		}
 		// break the net connection
 		NetConn_Close(host_client->netconnection);
@@ -457,6 +493,18 @@ void SV_DropClient(qboolean crash)
 	// update server listing on the master because player count changed
 	// (which the master uses for filtering empty/full servers)
 	NetConn_Heartbeat(1);
+
+	if (sv.loadgame)
+	{
+		for (i = 0;i < svs.maxclients;i++)
+			if (svs.clients[i].active && !svs.clients[i].spawned)
+				break;
+		if (i == svs.maxclients)
+		{
+			Con_Printf("Loaded game, everyone rejoined - unpausing\n");
+			sv.paused = sv.loadgame = false; // we're basically done with loading now
+		}
+	}
 }
 
 /*
@@ -480,6 +528,13 @@ void Host_ShutdownServer(void)
 
 // make sure all the clients know we're disconnecting
 	SV_VM_Begin();
+	if(prog->loaded)
+		if(prog->funcoffsets.SV_Shutdown)
+		{
+			func_t s = prog->funcoffsets.SV_Shutdown;
+			prog->funcoffsets.SV_Shutdown = 0; // prevent it from getting called again
+			PRVM_ExecuteProgram(s,"SV_Shutdown() required");
+		}
 	for (i = 0, host_client = svs.clients;i < svs.maxclients;i++, host_client++)
 		if (host_client->active)
 			SV_DropClient(false); // server shutdown
@@ -518,6 +573,18 @@ void Host_GetConsoleCommands (void)
 	}
 }
 
+/*
+==================
+Host_TimeReport
+
+Returns a time report string, for example for
+==================
+*/
+const char *Host_TimingReport()
+{
+	return va("%.1f%% CPU, %.2f%% lost, offset avg %.1fms, max %.1fms, sdev %.1fms", svs.perf_cpuload * 100, svs.perf_lost * 100, svs.perf_offset_avg * 1000, svs.perf_offset_max * 1000, svs.perf_offset_sdev * 1000);
+}
+
 /*
 ==================
 Host_Frame
@@ -534,7 +601,7 @@ void Host_Main(void)
 	double cl_timer, sv_timer;
 	double clframetime, deltarealtime, oldrealtime;
 	double wait;
-	int pass1, pass2, pass3;
+	int pass1, pass2, pass3, i;
 
 	Host_Init();
 
@@ -554,12 +621,38 @@ void Host_Main(void)
 		cl_timer += deltarealtime;
 		sv_timer += deltarealtime;
 
-		if (slowmo.value < 0)
+		svs.perf_acc_realtime += deltarealtime;
+
+		// Look for clients who have spawned
+		for (i = 0, host_client = svs.clients;i < svs.maxclients;i++, host_client++)
+			if(host_client->spawned)
+				if(host_client->netconnection)
+					break;
+		if(i == svs.maxclients)
+		{
+			// Nobody is looking? Then we won't do timing...
+			// Instead, reset it to zero
+			svs.perf_acc_realtime = svs.perf_acc_sleeptime = svs.perf_acc_lost = svs.perf_acc_offset = svs.perf_acc_offset_squared = svs.perf_acc_offset_max = svs.perf_acc_offset_samples = 0;
+		}
+		else if(svs.perf_acc_realtime > 5)
+		{
+			svs.perf_cpuload = 1 - svs.perf_acc_sleeptime / svs.perf_acc_realtime;
+			svs.perf_lost = svs.perf_acc_lost / svs.perf_acc_realtime;
+			if(svs.perf_acc_offset_samples > 0)
+			{
+				svs.perf_offset_max = svs.perf_acc_offset_max;
+				svs.perf_offset_avg = svs.perf_acc_offset / svs.perf_acc_offset_samples;
+				svs.perf_offset_sdev = sqrt(svs.perf_acc_offset_squared / svs.perf_acc_offset_samples - svs.perf_offset_avg * svs.perf_offset_avg);
+			}
+			if(svs.perf_lost > 0)
+				Con_DPrintf("Server can't keep up: %s\n", Host_TimingReport());
+			svs.perf_acc_realtime = svs.perf_acc_sleeptime = svs.perf_acc_lost = svs.perf_acc_offset = svs.perf_acc_offset_squared = svs.perf_acc_offset_max = svs.perf_acc_offset_samples = 0;
+		}
+
+		if (slowmo.value < 0.00001 && slowmo.value != 0)
 			Cvar_SetValue("slowmo", 0);
 		if (host_framerate.value < 0.00001 && host_framerate.value != 0)
 			Cvar_SetValue("host_framerate", 0);
-		if (cl_maxfps.value < 1)
-			Cvar_SetValue("cl_maxfps", 1);
 
 		// keep the random time dependent, but not when playing demos/benchmarking
 		if(!*sv_random_seed.string && !cls.demoplayback)
@@ -572,6 +665,8 @@ void Host_Main(void)
 
 		NetConn_UpdateSockets();
 
+		Log_DestBuffer_Flush();
+
 		// receive packets on each main loop iteration, as the main loop may
 		// be undersleeping due to select() detecting a new packet
 		if (sv.active)
@@ -600,20 +695,16 @@ void Host_Main(void)
 			wait = cl_timer * -1000000.0;
 		else
 			wait = max(cl_timer, sv_timer) * -1000000.0;
-		if (wait > 100000)
-			wait = 100000;
-		if (!cls.timedemo && wait > 0)
+		wait = bound(0, wait, 100000);
+
+		if (!cls.timedemo && wait >= 1)
 		{
+			double time0 = Sys_DoubleTime();
 			if (sv_checkforpacketsduringsleep.integer)
-			{
-				if (wait >= 1)
-					NetConn_SleepMicroseconds((int)wait);
-			}
+				NetConn_SleepMicroseconds((int)wait);
 			else
-			{
-				if (wait >= 1000)
-					Sys_Sleep((int)wait / 1000);
-			}
+				Sys_Sleep((int)wait);
+			svs.perf_acc_sleeptime += Sys_DoubleTime() - time0;
 			continue;
 		}
 
@@ -627,7 +718,10 @@ void Host_Main(void)
 		if (cl_timer > 0.1)
 			cl_timer = 0.1;
 		if (sv_timer > 0.1)
+		{
+			svs.perf_acc_lost += (sv_timer - 0.1);
 			sv_timer = 0.1;
+		}
 
 		if (sv.active && sv_timer > 0)
 		{
@@ -637,6 +731,7 @@ void Host_Main(void)
 			// slow down if the server is taking too long.
 			int framecount, framelimit = 1;
 			double advancetime, aborttime = 0;
+			float offset;
 
 			// run the world state
 			// don't allow simulation to run too fast or too slow or logic glitches can occur
@@ -663,6 +758,16 @@ void Host_Main(void)
 			}
 			advancetime = min(advancetime, 0.1);
 
+			if(advancetime > 0)
+			{
+				offset = sv_timer + (Sys_DoubleTime() - realtime);
+				++svs.perf_acc_offset_samples;
+				svs.perf_acc_offset += offset;
+				svs.perf_acc_offset_squared += offset * offset;
+				if(svs.perf_acc_offset_max < offset)
+					svs.perf_acc_offset_max = offset;
+			}
+
 			// only advance time if not paused
 			// the game also pauses in singleplayer when menu or console is used
 			sv.frametime = advancetime * slowmo.value;
@@ -717,9 +822,9 @@ void Host_Main(void)
 				}
 			}
 			else if (vid_activewindow)
-				clframetime = cl.realframetime = max(cl_timer, 1.0 / cl_maxfps.value);
+				clframetime = cl.realframetime = max(cl_timer, 1.0 / max(5.0f, cl_maxfps.value));
 			else
-				clframetime = cl.realframetime = 0.1;
+				clframetime = cl.realframetime = max(cl_timer, 1.0 / max(5.0f, cl_maxidlefps.value));
 
 			// apply slowmo scaling
 			clframetime *= cl.movevars_timescale;
@@ -766,7 +871,8 @@ void Host_Main(void)
 
 			//ui_update();
 
-			CL_VideoFrame();
+			CL_Video_Frame();
+			CL_Gecko_Frame();
 
 			CL_UpdateScreen();
 
@@ -780,7 +886,7 @@ void Host_Main(void)
 				cl.csqc_usecsqclistener = false;
 			}
 			else
-				S_Update(&r_view.matrix);
+				S_Update(&r_refdef.view.matrix);
 
 			CDAudio_Update();
 
@@ -799,7 +905,10 @@ void Host_Main(void)
 		if (cl_timer >= 0)
 			cl_timer = 0;
 		if (sv_timer >= 0)
+		{
+			svs.perf_acc_lost += sv_timer;
 			sv_timer = 0;
+		}
 
 		host_framecount++;
 	}
@@ -841,6 +950,9 @@ static void Host_Init (void)
 	int i;
 	const char* os;
 
+	if (COM_CheckParm("-profilegameonly"))
+		Sys_AllowProfiling(false);
+
 	// LordHavoc: quake never seeded the random number generator before... heh
 	if (COM_CheckParm("-benchmark"))
 		srand(0); // predictable random sequence for -benchmark
@@ -851,8 +963,8 @@ static void Host_Init (void)
 // COMMANDLINEOPTION: Console: -developer enables warnings and other notices (RECOMMENDED for mod developers)
 	if (COM_CheckParm("-developer"))
 	{
-		developer.value = developer.integer = 100;
-		developer.string = "100";
+		developer.value = developer.integer = 1;
+		developer.string = "1";
 	}
 
 	if (COM_CheckParm("-developer2"))
@@ -895,23 +1007,7 @@ static void Host_Init (void)
 	COM_InitGameType();
 
 	// construct a version string for the corner of the console
-#if defined(__linux__)
-	os = "Linux";
-#elif defined(WIN32)
-	os = "Windows";
-#elif defined(__FreeBSD__)
-	os = "FreeBSD";
-#elif defined(__NetBSD__)
-	os = "NetBSD";
-#elif defined(__OpenBSD__)
-	os = "OpenBSD";
-#elif defined(MACOSX)
-	os = "Mac OS X";
-#elif defined(__MORPHOS__)
-	os = "MorphOS";
-#else
-	os = "Unknown";
-#endif
+	os = DP_OS_NAME;
 	dpsnprintf (engineversion, sizeof (engineversion), "%s %s %s", gamename, os, buildstring);
 	Con_Printf("%s\n", engineversion);
 
@@ -935,7 +1031,7 @@ static void Host_Init (void)
 
 	if (cls.state != ca_dedicated)
 	{
-		Con_Printf("Initializing client\n");
+		Con_DPrintf("Initializing client\n");
 
 		R_Modules_Init();
 		Palette_Init();
@@ -1023,10 +1119,7 @@ static void Host_Init (void)
 
 	if (!sv.active && !cls.demoplayback && !cls.connect_trying)
 	{
-		if (gamemode == GAME_NEXUIZ)
-			Cbuf_AddText("togglemenu\nplayvideo logo\ncd loop 1\n");
-		else
-			Cbuf_AddText("togglemenu\n");
+		Cbuf_AddText("togglemenu\n");
 		Cbuf_Execute();
 	}
 
@@ -1076,9 +1169,10 @@ void Host_Shutdown(void)
 	// AK shutdown PRVM
 	// AK hmm, no PRVM_Shutdown(); yet
 
+	CL_Gecko_Shutdown();
 	CL_Video_Shutdown();
 
-	Host_SaveConfig_f();
+	Host_SaveConfig();
 
 	CDAudio_Shutdown ();
 	S_Terminate ();
