@@ -767,6 +767,15 @@ void Host_Name_f (void)
 			host_client->name[j++] = host_client->name[i];
 	host_client->name[j] = 0;
 
+	if(host_client->name[0] == 1 || host_client->name[0] == 2)
+	// may interfere with chat area, and will needlessly beep; so let's add a ^7
+	{
+		memmove(host_client->name + 2, host_client->name, sizeof(host_client->name) - 2);
+		host_client->name[sizeof(host_client->name) - 1] = 0;
+		host_client->name[0] = STRING_COLOR_TAG;
+		host_client->name[1] = '0' + STRING_COLOR_DEFAULT;
+	}
+
 	COM_StringLengthNoColors(host_client->name, 0, &valid_colors);
 	if(!valid_colors) // NOTE: this also proves the string is not empty, as "" is a valid colored string
 	{
@@ -1812,7 +1821,7 @@ Host_Viewmodel_f
 void Host_Viewmodel_f (void)
 {
 	prvm_edict_t	*e;
-	model_t	*m;
+	dp_model_t	*m;
 
 	if (!sv.active)
 		return;
@@ -1843,7 +1852,7 @@ void Host_Viewframe_f (void)
 {
 	prvm_edict_t	*e;
 	int		f;
-	model_t	*m;
+	dp_model_t	*m;
 
 	if (!sv.active)
 		return;
@@ -1863,7 +1872,7 @@ void Host_Viewframe_f (void)
 }
 
 
-void PrintFrameName (model_t *m, int frame)
+void PrintFrameName (dp_model_t *m, int frame)
 {
 	if (m->animscenes)
 		Con_Printf("frame %i: %s\n", frame, m->animscenes[frame].name);
@@ -1879,7 +1888,7 @@ Host_Viewnext_f
 void Host_Viewnext_f (void)
 {
 	prvm_edict_t	*e;
-	model_t	*m;
+	dp_model_t	*m;
 
 	if (!sv.active)
 		return;
@@ -1906,7 +1915,7 @@ Host_Viewprev_f
 void Host_Viewprev_f (void)
 {
 	prvm_edict_t	*e;
-	model_t	*m;
+	dp_model_t	*m;
 
 	if (!sv.active)
 		return;
