@@ -21,7 +21,7 @@
 		59 Temple Place - Suite 330
 		Boston, MA  02111-1307, USA
 
-	$Id: keys.h 5848 2005-11-28 07:46:39Z havoc $
+	$Id: keys.h 7897 2008-01-04 12:35:35Z divverent $
 */
 
 #ifndef __KEYS_H
@@ -50,6 +50,7 @@ typedef enum keynum_e
 	K_ALT,
 	K_CTRL,
 	K_SHIFT,
+
 	K_F1,
 	K_F2,
 	K_F3,
@@ -62,6 +63,7 @@ typedef enum keynum_e
 	K_F10,
 	K_F11,
 	K_F12,
+
 	K_INS,
 	K_DEL,
 	K_PGDN,
@@ -107,6 +109,7 @@ typedef enum keynum_e
 	// mouse buttons generate virtual keys
 
 	K_MOUSE1 = 512,
+	K_OTHERDEVICESBEGIN = K_MOUSE1,
 	K_MOUSE2,
 	K_MOUSE3,
 	K_MWHEELUP,
@@ -185,7 +188,7 @@ typedef enum keynum_e
 }
 keynum_t;
 
-typedef enum keydest_e { key_game, key_message, key_menu } keydest_t;
+typedef enum keydest_e { key_game, key_message, key_menu, key_menu_grabbed } keydest_t;
 
 #define MAX_INPUTLINES 32
 #define MAX_BINDMAPS 8
