@@ -6,7 +6,7 @@ static cvar_t cl_curl_maxdownloads = {CVAR_SAVE, "cl_curl_maxdownloads","1", "ma
 static cvar_t cl_curl_maxspeed = {CVAR_SAVE, "cl_curl_maxspeed","100", "maximum download speed (KiB/s)"};
 static cvar_t sv_curl_defaulturl = {CVAR_SAVE, "sv_curl_defaulturl","", "default autodownload source URL"};
 static cvar_t sv_curl_serverpackages = {CVAR_SAVE, "sv_curl_serverpackages","", "list of required files for the clients, separated by spaces"};
-static cvar_t cl_curl_enabled = {CVAR_SAVE, "cl_curl_enabled","0", "whether client's download support is enabled"};
+static cvar_t cl_curl_enabled = {CVAR_SAVE, "cl_curl_enabled","1", "whether client's download support is enabled"};
 
 /*
 =================================================================
@@ -330,14 +330,7 @@ static qboolean CURL_OpenLibrary (void)
 		return true;
 
 	// Load the DLL
-	if (! Sys_LoadLibrary (dllnames, &curl_dll, curlfuncs))
-	{
-		Con_Printf ("cURL support disabled\n");
-		return false;
-	}
-
-	Con_Printf ("cURL support enabled\n");
-	return true;
+	return Sys_LoadLibrary (dllnames, &curl_dll, curlfuncs);
 }
 
 
@@ -1276,7 +1269,7 @@ void Curl_ClearRequirements()
 	}
 	p = sv_curl_serverpackages.string;
 	Con_DPrintf("Require all of: %s\n", p);
-	while(COM_ParseToken_Simple(&p, false))
+	while(COM_ParseToken_Simple(&p, false, false))
 	{
 		Con_DPrintf("Require: %s\n", com_token);
 		Curl_RequireFile(com_token);
@@ -1298,7 +1291,6 @@ This is done by sending him the following console commands:
 void Curl_SendRequirements()
 {
 	// for each requirement, find the pack name
-	char datapatch[22]; // datapatchXXXXXXXX.pk3
 	char sendbuffer[4096] = "";
 	requirement *req;
 	qboolean foundone = false;
@@ -1316,26 +1308,6 @@ void Curl_SendRequirements()
 		if(p)
 			thispack = p + 1;
 
-		if(gamemode == GAME_NEXUIZ)
-		{
-			int y1, y2, y3, y4, m1, m2, d1, d2;
-			char eoftest, threetest;
-			if(9 == sscanf(thispack, "data%1d%1d%1d%1d%1d%1d%1d%1d.pk%c%c", &y1, &y2, &y3, &y4, &m1, &m2, &d1, &d2, &threetest, &eoftest))
-				// 9: the %c field did NOT MATCH! Note that %c always matches except the string is over...
-				if(threetest == '3') // but the pk did match
-				{
-					int y = y1 * 1000 + y2 * 100 + y3 * 10 + y4;
-					int m = m1 * 10 + m2;
-					int d = d1 * 10 + d2;
-					if(y >= 2007 && m >= 1 && m <= 12 && d >= 1 && d <= 31)
-					{
-						// replace data by datapatch
-						dpsnprintf(datapatch, sizeof(datapatch), "datapatch%04d%02d%02d.pk3", y, m, d);
-						thispack = datapatch;
-					}
-				}
-		}
-
 		packurl = Curl_FindPackURL(thispack);
 
 		if(packurl && *packurl && strcmp(packurl, "-"))
