@@ -9,11 +9,17 @@ entity_state_t defaultstate =
 	{0,0,0},//float netcenter[3]; // ! for network prioritization, this is the center of the bounding box (which may differ from the origin)
 	{0,0,0},//float origin[3];
 	{0,0,0},//float angles[3];
-	0,//int number; // entity number this state is for
 	0,//int effects;
+	0,//unsigned int customizeentityforclient; // !
+	0,//unsigned short number; // entity number this state is for
 	0,//unsigned short modelindex;
 	0,//unsigned short frame;
 	0,//unsigned short tagentity;
+	0,//unsigned short specialvisibilityradius; // ! larger if it has effects/light
+	0,//unsigned short viewmodelforclient; // !
+	0,//unsigned short exteriormodelforclient; // ! not shown if first person viewing from this entity, shown in all other cases
+	0,//unsigned short nodrawtoclient; // !
+	0,//unsigned short drawonlytoclient; // !
 	{0,0,0,0},//unsigned short light[4]; // color*256 (0.00 to 255.996), and radius*1
 	0,//unsigned char active; // true if a valid state
 	0,//unsigned char lightstyle;
@@ -29,7 +35,7 @@ entity_state_t defaultstate =
 	0,//unsigned char tagindex;
 	{32, 32, 32},//unsigned char colormod[3];
 	// padding to a multiple of 8 bytes (to align the double time)
-	0//unsigned char unused; // !
+	{0,0,0,0,0}//unsigned char unused[5]; // !
 };
 
 // LordHavoc: I own protocol ranges 96, 97, 3500-3599
@@ -53,6 +59,9 @@ protocolversioninfo[] =
 	{250, "NEHAHRAMOVIE"},
 	{15, "QUAKE"},
 	{28, "QW"},
+	{10000, "NEHAHRABJP"},
+	{10001, "NEHAHRABJP2"},
+	{10002, "NEHAHRABJP3"},
 	{0, NULL}
 };
 
@@ -149,7 +158,13 @@ void EntityFrameQuake_ReadEntity(int bits)
 	s.number = num;
 	s.time = cl.mtime[0];
 	s.flags = 0;
-	if (bits & U_MODEL)		s.modelindex = (s.modelindex & 0xFF00) | MSG_ReadByte();
+	if (bits & U_MODEL)
+	{
+		if (cls.protocol == PROTOCOL_NEHAHRABJP || cls.protocol == PROTOCOL_NEHAHRABJP2 || cls.protocol == PROTOCOL_NEHAHRABJP3)
+							s.modelindex = (unsigned short) MSG_ReadShort();
+		else
+							s.modelindex = (s.modelindex & 0xFF00) | MSG_ReadByte();
+	}
 	if (bits & U_FRAME)		s.frame = (s.frame & 0xFF00) | MSG_ReadByte();
 	if (bits & U_COLORMAP)	s.colormap = MSG_ReadByte();
 	if (bits & U_SKIN)		s.skin = MSG_ReadByte();
@@ -285,10 +300,10 @@ void EntityFrameCSQC_WriteState (sizebuf_t *msg, int number, qboolean doupdate,
 			if(!*sectionstarted)
 				MSG_WriteByte(msg, svc_csqcentities);
 			MSG_WriteShort(msg, number);
-			((int *)prog->globals.generic)[OFS_PARM0] = sv.writeentitiestoclient_cliententitynumber;
+			PRVM_G_INT(OFS_PARM0) = sv.writeentitiestoclient_cliententitynumber;
 			prog->globals.server->self = number;
 			PRVM_ExecuteProgram(val->function, "Null SendEntity\n");
-			if(prog->globals.generic[OFS_RETURN])
+			if(PRVM_G_FLOAT(OFS_RETURN))
 			{
 				if (msg->cursize + 2 > msg->maxsize)
 				{
@@ -387,9 +402,28 @@ void Protocol_UpdateClientStats(const int *stats)
 	}
 }
 
+// only a few stats are within the 32 stat limit of Quake, and most of them
+// are sent every frame in svc_clientdata messages, so we only send the
+// remaining ones here
+static const int sendquakestats[] =
+{
+// quake did not send these secrets/monsters stats in this way, but doing so
+// allows a mod to increase STAT_TOTALMONSTERS during the game, and ensures
+// that STAT_SECRETS and STAT_MONSTERS are always correct (even if a client
+// didn't receive an svc_foundsecret or svc_killedmonster), which may be most
+// valuable if randomly seeking around in a demo
+STAT_TOTALSECRETS, // never changes during game
+STAT_TOTALMONSTERS, // changes in some mods
+STAT_SECRETS, // this makes svc_foundsecret unnecessary
+STAT_MONSTERS, // this makes svc_killedmonster unnecessary
+STAT_VIEWHEIGHT, // sent just for FTEQW clients
+STAT_VIEWZOOM, // this rarely changes
+-1,
+};
+
 void Protocol_WriteStatsReliable(void)
 {
-	int i;
+	int i, j;
 	if (!host_client->netconnection)
 		return;
 	// detect changes in stats and write reliable messages
@@ -397,14 +431,9 @@ void Protocol_WriteStatsReliable(void)
 	// this function can only cope with 32 stats,
 	// they also do not support svc_updatestatubyte which was introduced in
 	// DP6 protocol (except for QW)
-	for (i = 0;i < 32;i++)
+	for (j = 0;sendquakestats[j] >= 0;j++)
 	{
-		// quickly skip zero bytes
-		if (!host_client->statsdeltabits[i >> 3])
-		{
-			i |= 7;
-			continue;
-		}
+		i = sendquakestats[j];
 		// check if this bit is set
 		if (host_client->statsdeltabits[i >> 3] & (1 << (i & 7)))
 		{
@@ -507,7 +536,7 @@ void EntityFrameQuake_WriteFrame(sizebuf_t *msg, int numstates, const entity_sta
 		if (baseline.modelindex != s->modelindex)
 		{
 			bits |= U_MODEL;
-			if (s->modelindex & 0xFF00)
+			if ((s->modelindex & 0xFF00) && sv.protocol != PROTOCOL_NEHAHRABJP && sv.protocol != PROTOCOL_NEHAHRABJP2 && sv.protocol != PROTOCOL_NEHAHRABJP3)
 				bits |= U_MODEL2;
 		}
 		if (baseline.alpha != s->alpha)
@@ -539,12 +568,21 @@ void EntityFrameQuake_WriteFrame(sizebuf_t *msg, int numstates, const entity_sta
 
 		MSG_WriteByte (&buf, bits);
 		if (bits & U_MOREBITS)		MSG_WriteByte(&buf, bits>>8);
-		if (bits & U_EXTEND1)		MSG_WriteByte(&buf, bits>>16);
-		if (bits & U_EXTEND2)		MSG_WriteByte(&buf, bits>>24);
+		if (sv.protocol != PROTOCOL_NEHAHRAMOVIE)
+		{
+			if (bits & U_EXTEND1)	MSG_WriteByte(&buf, bits>>16);
+			if (bits & U_EXTEND2)	MSG_WriteByte(&buf, bits>>24);
+		}
 		if (bits & U_LONGENTITY)	MSG_WriteShort(&buf, s->number);
 		else						MSG_WriteByte(&buf, s->number);
 
-		if (bits & U_MODEL)			MSG_WriteByte(&buf, s->modelindex);
+		if (bits & U_MODEL)
+		{
+			if (sv.protocol == PROTOCOL_NEHAHRABJP || sv.protocol == PROTOCOL_NEHAHRABJP2 || sv.protocol == PROTOCOL_NEHAHRABJP3)
+				MSG_WriteShort(&buf, s->modelindex);
+			else
+				MSG_WriteByte(&buf, s->modelindex);
+		}
 		if (bits & U_FRAME)			MSG_WriteByte(&buf, s->frame);
 		if (bits & U_COLORMAP)		MSG_WriteByte(&buf, s->colormap);
 		if (bits & U_SKIN)			MSG_WriteByte(&buf, s->skin);
@@ -1767,8 +1805,17 @@ void EntityState5_WriteUpdate(int number, const entity_state_t *s, int changedbi
 	else
 	{
 		bits = changedbits;
-		if ((bits & E5_ORIGIN) && ((s->flags & RENDER_EXTERIORMODEL) || s->origin[0] < -4096 || s->origin[0] >= 4096 || s->origin[1] < -4096 || s->origin[1] >= 4096 || s->origin[2] < -4096 || s->origin[2] >= 4096))
+		if ((bits & E5_ORIGIN) && ((s->flags & RENDER_EXTERIORMODEL) || s->origin[0] <= -4096.0625 || s->origin[0] >= 4095.9375 || s->origin[1] <= -4096.0625 || s->origin[1] >= 4095.9375 || s->origin[2] <= -4096.0625 || s->origin[2] >= 4095.9375))
 			bits |= E5_ORIGIN32;
+			// possible values:
+			//   negative origin:
+			//     (int)(f * 8 - 0.5) >= -32768
+			//          (f * 8 - 0.5) >  -32769
+			//           f            >  -4096.0625
+			//   positive origin:
+			//     (int)(f * 8 + 0.5) <=  32767
+			//          (f * 8 + 0.5) <   32768
+			//           f * 8 + 0.5) <   4095.9375
 		if ((bits & E5_ANGLES) && !(s->flags & RENDER_LOWPRECISION))
 			bits |= E5_ANGLES16;
 		if ((bits & E5_MODEL) && s->modelindex >= 256)
@@ -2197,7 +2244,7 @@ void EntityFrame5_LostFrame(entityframe5_database_t *d, int framenum)
 				{
 					d->deltabits[s->number] |= bits;
 					// if it was a very important update, set priority higher
-					if (bits & (E5_FULLUPDATE | E5_ATTACHMENT | E5_MODEL || E5_COLORMAP))
+					if (bits & (E5_FULLUPDATE | E5_ATTACHMENT | E5_MODEL | E5_COLORMAP))
 						d->priorities[s->number] = max(d->priorities[s->number], 4);
 					else
 						d->priorities[s->number] = max(d->priorities[s->number], 1);
@@ -2340,7 +2387,7 @@ void EntityFrame5_WriteFrame(sizebuf_t *msg, entityframe5_database_t *d, int num
 	packetlog->packetnumber = framenum;
 	packetlog->numstates = 0;
 	// write stat updates
-	if (sv.protocol != PROTOCOL_QUAKE && sv.protocol != PROTOCOL_QUAKEDP && sv.protocol != PROTOCOL_NEHAHRAMOVIE && sv.protocol != PROTOCOL_DARKPLACES1 && sv.protocol != PROTOCOL_DARKPLACES2 && sv.protocol != PROTOCOL_DARKPLACES3 && sv.protocol != PROTOCOL_DARKPLACES4 && sv.protocol != PROTOCOL_DARKPLACES5)
+	if (sv.protocol != PROTOCOL_QUAKE && sv.protocol != PROTOCOL_QUAKEDP && sv.protocol != PROTOCOL_NEHAHRAMOVIE && sv.protocol != PROTOCOL_NEHAHRABJP && sv.protocol != PROTOCOL_NEHAHRABJP2 && sv.protocol != PROTOCOL_NEHAHRABJP3 && sv.protocol != PROTOCOL_DARKPLACES1 && sv.protocol != PROTOCOL_DARKPLACES2 && sv.protocol != PROTOCOL_DARKPLACES3 && sv.protocol != PROTOCOL_DARKPLACES4 && sv.protocol != PROTOCOL_DARKPLACES5)
 	{
 		for (i = 0;i < MAX_CL_STATS && msg->cursize + 6 + 11 <= msg->maxsize;i++)
 		{
