@@ -88,6 +88,7 @@ float	gettime()
 		parseentitydata(entity ent, string data)
 float	mod(float val, float m)
 const string	cvar_string (string)
+float	cvar_type (string)
 		crash()
 		stackdump()
 
@@ -126,7 +127,7 @@ perhaps only : Menu : WriteMsg
 		WriteString(string data, float dest, float desto)
 		WriteEntity(entity data, float dest, float desto)
 
-Client & Menu : draw functions & video functions
+Client & Menu : draw functions & video functions (& gecko functions)
 ===================================================
 
 float	iscachedpic(string pic)
@@ -134,7 +135,10 @@ string	precache_pic(string pic)
 		freepic(string s)
 float	drawcharacter(vector position, float character, vector scale, vector rgb, float alpha, float flag)
 float	drawstring(vector position, string text, vector scale, vector rgb, float alpha, float flag)
+float	drawcolorcodedstring(vector position, string text, vector scale, float alpha, float flag)
+float	stringwidth(string text, float handleColors)
 float	drawpic(vector position, string pic, vector size, vector rgb, float alpha, float flag)
+float	drawsubpic(vector position, vector size, string pic, vector srcPos, vector srcSize, vector rgb, float alpha, float flag)
 float	drawfill(vector position, vector size, vector rgb, float alpha, float flag)
 		drawsetcliparea(float x, float y, float width, float height)
 		drawresetcliparea()
@@ -146,6 +150,12 @@ void	cin_setstate(string name, float type)
 float	cin_getstate(string name)
 void	cin_restart(string name)
 
+float[bool] gecko_create( string name )
+void gecko_destroy( string name )
+void gecko_navigate( string name, string URI )
+float[bool] gecko_keyevent( string name, float key, float eventtype ) 
+void gecko_mousemove( string name, float x, float y )
+
 ==============================================================================
 menu cmd list:
 ===============
@@ -184,6 +194,7 @@ float	getserverlistindexforkey(string key)
 #include "mprogdefs.h"
 
 #include "cl_video.h"
+#include "cl_gecko.h"
 
 //============================================================================
 // nice helper macros
@@ -222,6 +233,7 @@ void VM_break (void);
 void VM_localcmd (void);
 void VM_cvar (void);
 void VM_cvar_string(void);
+void VM_cvar_type (void);
 void VM_cvar_defstring (void);
 void VM_cvar_set (void);
 void VM_dprint (void);
@@ -323,7 +335,10 @@ void VM_precache_pic(void);
 void VM_freepic(void);
 void VM_drawcharacter(void);
 void VM_drawstring(void);
+void VM_drawcolorcodedstring(void);
+void VM_stringwidth(void);
 void VM_drawpic(void);
+void VM_drawsubpic(void);
 void VM_drawfill(void);
 void VM_drawsetcliparea(void);
 void VM_drawresetcliparea(void);
@@ -341,6 +356,14 @@ void VM_cin_setstate( void );
 void VM_cin_getstate( void );
 void VM_cin_restart( void );
 
+void VM_gecko_create( void );
+void VM_gecko_destroy( void );
+void VM_gecko_navigate( void );
+void VM_gecko_keyevent( void );
+void VM_gecko_movemouse( void );
+void VM_gecko_resize( void );
+void VM_gecko_get_texture_extent( void );
+
 void VM_drawline (void);
 
 void VM_bitshift (void);
@@ -366,13 +389,31 @@ void VM_changeyaw (void);
 void VM_changepitch (void);
 
 void VM_uncolorstring (void);
+
+void VM_strstrofs (void);
 void VM_str2chr (void);
 void VM_chr2str (void);
+void VM_strconv (void);
+void VM_strpad (void);
+void VM_infoadd (void);
+void VM_infoget (void);
 void VM_strncmp (void);
+void VM_strncmp (void);
+void VM_strncasecmp (void);
 void VM_registercvar (void);
 void VM_wasfreed (void);
 
+void VM_strreplace (void);
+void VM_strireplace (void);
+
+void VM_crc16(void);
+
 void VM_SetTraceGlobals(const trace_t *trace);
 
 void VM_Cmd_Init(void);
 void VM_Cmd_Reset(void);
+
+void VM_uri_escape (void);
+void VM_uri_unescape (void);
+
+void VM_etof (void);
