@@ -22,6 +22,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef QUAKEDEF_H
 #define QUAKEDEF_H
 
+// on UNIX platforms we need to define this so that video saving does not cause a SIGFSZ (file size) signal when a video clip exceeds 2GB
+#define _FILE_OFFSET_BITS 64
+
+// for cd_linux.c
+#define __KERNEL_STRICT_NAMES
+
 #if defined(__GNUC__) && (__GNUC__ > 2)
 #define DP_FUNC_PRINTF(n) __attribute__ ((format (printf, n, n+1)))
 #else
@@ -76,7 +82,7 @@ extern char engineversion[128];
 //
 #define	MAX_CL_STATS		256
 #define	STAT_HEALTH			0
-#define	STAT_FRAGS			1
+//#define	STAT_FRAGS			1
 #define	STAT_WEAPON			2
 #define	STAT_AMMO			3
 #define	STAT_ARMOR			4
@@ -95,6 +101,8 @@ extern char engineversion[128];
 //#define STAT_TIME			17 // FTE
 //#define STAT_VIEW2		20 // FTE
 #define STAT_VIEWZOOM		21 // DP
+#define STAT_FRAGLIMIT								235 // DP
+#define STAT_TIMELIMIT								236 // DP
 #define STAT_MOVEVARS_WALLFRICTION					237 // DP
 #define STAT_MOVEVARS_FRICTION						238 // DP
 #define STAT_MOVEVARS_WATERFRICTION					239 // DP
@@ -251,8 +259,56 @@ extern char engineversion[128];
 
 extern qboolean noclip_anglehack;
 
-extern char engineversion[128];
 extern cvar_t developer;
+extern cvar_t developer_loadfile;
+extern cvar_t developer_loading;
+
+/* Preprocessor macros to identify platform
+    DP_OS_NAME 	- "friendly" name of the OS, for humans to read
+    DP_OS_STR	- "identifier" of the OS, more suited for code to use
+    DP_ARCH_STR	- "identifier" of the processor architecture
+ */
+#if defined(__linux__)
+# define DP_OS_NAME		"Linux"
+# define DP_OS_STR		"linux"
+#elif defined(WIN64)
+# define DP_OS_NAME		"Windows64"
+# define DP_OS_STR		"win64"
+#elif defined(WIN32)
+# define DP_OS_NAME		"Windows"
+# define DP_OS_STR		"win32"
+#elif defined(__FreeBSD__)
+# define DP_OS_NAME		"FreeBSD"
+# define DP_OS_STR		"freebsd"
+#elif defined(__NetBSD__)
+# define DP_OS_NAME		"NetBSD"
+# define DP_OS_STR		"netbsd"
+#elif defined(__OpenBSD__)
+# define DP_OS_NAME		"OpenBSD"
+# define DP_OS_STR		"openbsd"
+#elif defined(MACOSX)
+# define DP_OS_NAME		"Mac OS X"
+# define DP_OS_STR		"osx"
+#elif defined(__MORPHOS__)
+# define DP_OS_NAME		"MorphOS"
+# define DP_OS_STR		"morphos"
+#else
+# define DP_OS_NAME		"Unknown"
+#endif
+
+#if defined(__GNUC__)
+# if defined(__i386__)
+#  define DP_ARCH_STR		"686"
+# elif defined(__x86_64__)
+#  define DP_ARCH_STR		"x86_64"
+# elif defined(__powerpc__)
+#  define DP_ARCH_STR		"ppc"
+# endif
+#elif defined(WIN64)
+# define DP_ARCH_STR		"x86_64"
+#elif defined(WIN32)
+# define DP_ARCH_STR		"x86"
+#endif
 
 // incremented every frame, never reset
 extern int host_framecount;
@@ -268,6 +324,7 @@ void Host_Quit_f(void);
 void Host_ClientCommands(const char *fmt, ...) DP_FUNC_PRINTF(1);
 void Host_ShutdownServer(void);
 void Host_Reconnect_f(void);
+void Host_NoOperation_f(void);
 
 void Host_AbortCurrentFrame(void);
 
@@ -290,5 +347,10 @@ float noise4f(float x, float y, float z, float w);
 
 void Sys_Shared_Init(void);
 
+// Flag in size field of demos to indicate a client->server packet. Demo
+// playback will ignore this, but it may be useful to make DP sniff packets to
+// debug protocol exploits.
+#define DEMOMSG_CLIENT_TO_SERVER 0x80000000
+
 #endif
 
