@@ -43,10 +43,8 @@ extern void R_ResetSkyBox(void);
 extern void SHOWLMP_decodehide(void);
 extern void SHOWLMP_decodeshow(void);
 extern void SHOWLMP_drawall(void);
-extern void SHOWLMP_clear(void);
 
 // render profiling stuff
-extern char r_speeds_string[1024];
 extern int r_timereport_active;
 
 // lighting stuff
@@ -58,6 +56,7 @@ extern cvar_t r_novis;
 
 extern cvar_t r_lerpsprites;
 extern cvar_t r_lerpmodels;
+extern cvar_t r_lerplightstyles;
 extern cvar_t r_waterscroll;
 
 extern cvar_t developer_texturelogging;
@@ -125,18 +124,28 @@ void R_Init(void);
 void R_UpdateVariables(void); // must call after setting up most of r_refdef, but before calling R_RenderView
 void R_RenderView(void); // must set r_refdef and call R_UpdateVariables first
 
+typedef enum r_refdef_scene_type_s {
+	RST_CLIENT,
+	RST_MENU,
+	RST_COUNT
+} r_refdef_scene_type_t;
 
-void R_InitSky (unsigned char *src, int bytesperpixel); // called at level load
+void R_SelectScene( r_refdef_scene_type_t scenetype );
+r_refdef_scene_t * R_GetScenePointer( r_refdef_scene_type_t scenetype );
 
 void R_SkinFrame_PrepareForPurge(void);
 void R_SkinFrame_MarkUsed(skinframe_t *skinframe);
 void R_SkinFrame_Purge(void);
+// set last to NULL to start from the beginning
+skinframe_t *R_SkinFrame_FindNextByName( skinframe_t *last, const char *name );
 skinframe_t *R_SkinFrame_Find(const char *name, int textureflags, int comparewidth, int compareheight, int comparecrc, qboolean add);
 skinframe_t *R_SkinFrame_LoadExternal(const char *name, int textureflags, qboolean complain);
-skinframe_t *R_SkinFrame_LoadInternal(const char *name, int textureflags, int loadpantsandshirt, int loadglowtexture, const unsigned char *skindata, int width, int height, int bitsperpixel, const unsigned int *palette, const unsigned int *alphapalette);
+skinframe_t *R_SkinFrame_LoadInternalBGRA(const char *name, int textureflags, const unsigned char *skindata, int width, int height);
+skinframe_t *R_SkinFrame_LoadInternalQuake(const char *name, int textureflags, int loadpantsandshirt, int loadglowtexture, const unsigned char *skindata, int width, int height);
 skinframe_t *R_SkinFrame_LoadMissing(void);
 
-void R_View_WorldVisibility();
+void R_View_WorldVisibility(qboolean forcenovis);
+void R_DrawDecals(void);
 void R_DrawParticles(void);
 void R_DrawExplosions(void);
 
@@ -171,10 +180,9 @@ extern cvar_t r_test;
 
 #include "gl_backend.h"
 
-#include "r_light.h"
-
 extern rtexture_t *r_texture_blanknormalmap;
 extern rtexture_t *r_texture_white;
+extern rtexture_t *r_texture_grey128;
 extern rtexture_t *r_texture_black;
 extern rtexture_t *r_texture_notexture;
 extern rtexture_t *r_texture_whitecube;
@@ -192,16 +200,6 @@ void R_DrawSprite(int blendfunc1, int blendfunc2, rtexture_t *texture, rtexture_
 
 extern mempool_t *r_main_mempool;
 
-typedef enum rsurfmode_e
-{
-	RSURFMODE_NONE,
-	RSURFMODE_SHOWSURFACES,
-	RSURFMODE_SKY,
-	RSURFMODE_MULTIPASS,
-	RSURFMODE_GLSL
-}
-rsurfmode_t;
-
 typedef struct rsurfacestate_s
 {
 	// processing buffers
@@ -255,7 +253,9 @@ typedef struct rsurfacestate_s
 	int modeltexcoordlightmap2f_bufferobject;
 	size_t modeltexcoordlightmap2f_bufferoffset;
 	int *modelelement3i;
+	unsigned short *modelelement3s;
 	int modelelement3i_bufferobject;
+	int modelelement3s_bufferobject;
 	int *modellightmapoffsets;
 	int modelnum_vertices;
 	int modelnum_triangles;
@@ -305,15 +305,14 @@ typedef struct rsurfacestate_s
 	vec3_t colormap_shirtcolor;
 	// view location in model space
 	vec3_t modelorg; // TODO: rename this
+	// polygon offset data for submodels
+	float basepolygonfactor;
+	float basepolygonoffset;
 	// current texture in batching code
 	texture_t *texture;
 	// whether lightmapping is active on this batch
 	// (otherwise vertex colored)
 	qboolean uselightmaptexture;
-	// one of the RSURFMODE_ values
-	rsurfmode_t mode;
-	// type of vertex lighting being used on this batch
-	int lightmode; // 0 = lightmap or fullbright, 1 = color array from q3bsp, 2 = vertex shaded model
 
 	// rtlight rendering
 	// light currently being rendered
@@ -342,7 +341,7 @@ extern rsurfacestate_t rsurface;
 
 void RSurf_ActiveWorldEntity(void);
 void RSurf_ActiveModelEntity(const entity_render_t *ent, qboolean wantnormals, qboolean wanttangents);
-void RSurf_CleanUp(void);
+void RSurf_SetupDepthAndCulling(void);
 
 void R_Mesh_ResizeArrays(int newvertices);
 
@@ -352,81 +351,94 @@ struct msurface_s;
 void R_UpdateTextureInfo(const entity_render_t *ent, texture_t *t);
 void R_UpdateAllTextureInfo(entity_render_t *ent);
 void R_QueueTextureSurfaceList(int texturenumsurfaces, msurface_t **texturesurfacelist);
-void R_DrawWorldSurfaces(qboolean skysurfaces, qboolean writedepth, qboolean depthonly);
-void R_DrawModelSurfaces(entity_render_t *ent, qboolean skysurfaces, qboolean writedepth, qboolean depthonly);
+void R_DrawWorldSurfaces(qboolean skysurfaces, qboolean writedepth, qboolean depthonly, qboolean addwaterplanes, qboolean debug);
+void R_DrawModelSurfaces(entity_render_t *ent, qboolean skysurfaces, qboolean writedepth, qboolean depthonly, qboolean addwaterplanes, qboolean debug);
 
 void RSurf_PrepareVerticesForBatch(qboolean generatenormals, qboolean generatetangents, int texturenumsurfaces, msurface_t **texturesurfacelist);
 void RSurf_DrawBatch_Simple(int texturenumsurfaces, msurface_t **texturesurfacelist);
 
-#define SHADERPERMUTATION_MODE_LIGHTSOURCE (1<<0) // (lightsource) use directional pixel shading from light source (rtlight)
-#define SHADERPERMUTATION_MODE_LIGHTDIRECTIONMAP_MODELSPACE (1<<1) // (lightmap) use directional pixel shading from texture containing modelspace light directions (deluxemap)
-#define SHADERPERMUTATION_MODE_LIGHTDIRECTIONMAP_TANGENTSPACE (1<<2) // (lightmap) use directional pixel shading from texture containing tangentspace light directions (deluxemap)
-#define SHADERPERMUTATION_MODE_LIGHTDIRECTION (1<<3) // (lightmap) use directional pixel shading from fixed light direction (q3bsp)
-#define SHADERPERMUTATION_GLOW (1<<4) // (lightmap) blend in an additive glow texture
-#define SHADERPERMUTATION_FOG (1<<5) // tint the color by fog color or black if using additive blend mode
-#define SHADERPERMUTATION_COLORMAPPING (1<<6) // indicates this is a colormapped skin
-#define SHADERPERMUTATION_DIFFUSE (1<<7) // (lightsource) whether to use directional shading
-#define SHADERPERMUTATION_CONTRASTBOOST (1<<8) // r_glsl_contrastboost boosts the contrast at low color levels (similar to gamma)
-#define SHADERPERMUTATION_SPECULAR (1<<9) // (lightsource or deluxemapping) render specular effects
-#define SHADERPERMUTATION_CUBEFILTER (1<<10) // (lightsource) use cubemap light filter
-#define SHADERPERMUTATION_OFFSETMAPPING (1<<11) // adjust texcoords to roughly simulate a displacement mapped surface
-#define SHADERPERMUTATION_OFFSETMAPPING_RELIEFMAPPING (1<<12) // adjust texcoords to accurately simulate a displacement mapped surface (requires OFFSETMAPPING to also be set!)
-
-#define SHADERPERMUTATION_MAX (1<<13) // how many permutations are possible
-#define SHADERPERMUTATION_MASK (SHADERPERMUTATION_MAX - 1) // mask of valid indexing bits for r_glsl_permutations[] array
-
-// these are additional flags used only by R_GLSL_CompilePermutation
-#define SHADERPERMUTATION_USES_VERTEXSHADER (1<<29)
-#define SHADERPERMUTATION_USES_GEOMETRYSHADER (1<<30)
-#define SHADERPERMUTATION_USES_FRAGMENTSHADER (1<<31)
-
-typedef struct r_glsl_permutation_s
+typedef enum rsurfacepass_e
 {
-	// indicates if we have tried compiling this permutation already
-	qboolean compiled;
-	// 0 if compilation failed
-	int program;
-	int loc_Texture_Normal;
-	int loc_Texture_Color;
-	int loc_Texture_Gloss;
-	int loc_Texture_Cube;
-	int loc_Texture_Attenuation;
-	int loc_Texture_FogMask;
-	int loc_Texture_Pants;
-	int loc_Texture_Shirt;
-	int loc_Texture_Lightmap;
-	int loc_Texture_Deluxemap;
-	int loc_Texture_Glow;
-	int loc_FogColor;
-	int loc_LightPosition;
-	int loc_EyePosition;
-	int loc_LightColor;
-	int loc_Color_Pants;
-	int loc_Color_Shirt;
-	int loc_FogRangeRecip;
-	int loc_AmbientScale;
-	int loc_DiffuseScale;
-	int loc_SpecularScale;
-	int loc_SpecularPower;
-	int loc_GlowScale;
-	int loc_SceneBrightness; // or: Scenebrightness * ContrastBoost
-	int loc_OffsetMapping_Scale;
-	int loc_AmbientColor;
-	int loc_DiffuseColor;
-	int loc_SpecularColor;
-	int loc_LightDir;
-	int loc_ContrastBoostCoeff; // 1 - 1/ContrastBoost
+	RSURFPASS_BASE,
+	RSURFPASS_BACKGROUND,
+	RSURFPASS_RTLIGHT
 }
-r_glsl_permutation_t;
+rsurfacepass_t;
 
-// information about each possible shader permutation
-extern r_glsl_permutation_t r_glsl_permutations[SHADERPERMUTATION_MAX];
-// currently selected permutation
-extern r_glsl_permutation_t *r_glsl_permutation;
+typedef enum gl20_texunit_e
+{
+	// postprocess shaders, and generic shaders:
+	GL20TU_FIRST = 0,
+	GL20TU_SECOND = 1,
+	GL20TU_GAMMARAMPS = 2,
+	// standard material properties
+	GL20TU_NORMAL = 0,
+	GL20TU_COLOR = 1,
+	GL20TU_GLOSS = 2,
+	GL20TU_GLOW = 3,
+	// material properties for a second material
+	GL20TU_SECONDARY_NORMAL = 4,
+	GL20TU_SECONDARY_COLOR = 5,
+	GL20TU_SECONDARY_GLOSS = 6,
+	GL20TU_SECONDARY_GLOW = 7,
+	// material properties for a colormapped material
+	// conflicts with secondary material
+	GL20TU_PANTS = 4,
+	GL20TU_SHIRT = 5,
+	// fog fade in the distance
+	GL20TU_FOGMASK = 8,
+	// compiled ambient lightmap and deluxemap
+	GL20TU_LIGHTMAP = 9,
+	GL20TU_DELUXEMAP = 10,
+	// refraction, used by water shaders
+	GL20TU_REFRACTION = 3,
+	// reflection, used by water shaders, also with normal material rendering
+	// conflicts with secondary material
+	GL20TU_REFLECTION = 7,
+	// rtlight attenuation (distance fade) and cubemap filter (projection texturing)
+	// conflicts with lightmap/deluxemap
+	GL20TU_ATTENUATION = 9,
+	GL20TU_CUBE = 10,
+}
+gl20_texunit;
 
-void R_GLSL_CompilePermutation(const char *shaderfilename, int permutation);
-int R_SetupSurfaceShader(const vec3_t lightcolorbase, qboolean modellighting, float ambientscale, float diffusescale, float specularscale);
-void R_SwitchSurfaceShader(int permutation);
+void R_SetupGenericShader(qboolean usetexture);
+void R_SetupGenericTwoTextureShader(int texturemode);
+void R_SetupDepthOrShadowShader(void);
+void R_SetupSurfaceShader(const vec3_t lightcolorbase, qboolean modellighting, float ambientscale, float diffusescale, float specularscale, rsurfacepass_t rsurfacepass);
+
+typedef struct r_waterstate_waterplane_s
+{
+	rtexture_t *texture_refraction;
+	rtexture_t *texture_reflection;
+	mplane_t plane;
+	int materialflags; // combined flags of all water surfaces on this plane
+	unsigned char pvsbits[(32768+7)>>3]; // FIXME: buffer overflow on huge maps
+	qboolean pvsvalid;
+}
+r_waterstate_waterplane_t;
+
+#define MAX_WATERPLANES 16
+
+typedef struct r_waterstate_s
+{
+	qboolean enabled;
+
+	qboolean renderingscene; // true while rendering a refraction or reflection texture, disables water surfaces
+
+	int waterwidth, waterheight;
+	int texturewidth, textureheight;
+
+	int maxwaterplanes; // same as MAX_WATERPLANES
+	int numwaterplanes;
+	r_waterstate_waterplane_t waterplanes[MAX_WATERPLANES];
+
+	float screenscale[2];
+	float screencenter[2];
+}
+r_waterstate_t;
+
+extern r_waterstate_t r_waterstate;
 
 #endif
 
