@@ -34,6 +34,20 @@ typedef struct server_static_s
 	qboolean changelevel_issued;
 	// server infostring
 	char serverinfo[MAX_SERVERINFO_STRING];
+	// performance data
+	float perf_cpuload;
+	float perf_lost;
+	float perf_offset_avg;
+	float perf_offset_max;
+	float perf_offset_sdev;
+	// temporary performance data accumulators
+	float perf_acc_realtime;
+	float perf_acc_sleeptime;
+	float perf_acc_lost;
+	float perf_acc_offset;
+	float perf_acc_offset_squared;
+	float perf_acc_offset_max;
+	int perf_acc_offset_samples;
 } server_static_t;
 
 //=============================================================================
@@ -125,6 +139,14 @@ typedef struct server_s
 	int writeentitiestoclient_pvsbytes;
 	unsigned char writeentitiestoclient_pvs[MAX_MAP_LEAFS/8];
 	entity_state_t writeentitiestoclient_sendstates[MAX_EDICTS];
+
+	int numsendentities;
+	entity_state_t sendentities[MAX_EDICTS];
+	entity_state_t *sendentitiesindex[MAX_EDICTS];
+
+	int sententitiesmark;
+	int sententities[MAX_EDICTS];
+	int sententitiesconsideration[MAX_EDICTS];
 } server_t;
 
 // if defined this does ping smoothing, otherwise it does not
@@ -140,6 +162,8 @@ typedef struct client_s
 	qboolean clientconnectcalled;
 	// false = don't send datagrams
 	qboolean spawned;
+	// 1 = send svc_serverinfo and advance to 2, 2 doesn't send, then advances to 0 (allowing unlimited sending) when prespawn is received
+	int sendsignon;
 
 	// requested rate in bytes per second
 	int rate;
@@ -204,6 +228,9 @@ typedef struct client_s
 	char weaponmodel[MAX_QPATH];
 	int weaponmodelindex;
 
+	// clientcamera (entity to use as camera)
+	int clientcamera;
+
 	entityframe_database_t *entitydatabase;
 	entityframe4_database_t *entitydatabase4;
 	entityframe5_database_t *entitydatabase5;
@@ -226,6 +253,9 @@ typedef struct client_s
 	// fixangle data
 	qboolean fixangle_angles_set;
 	vec3_t fixangle_angles;
+
+	// demo recording
+	qfile_t *sv_demo_file;
 } client_t;
 
 
@@ -340,11 +370,14 @@ extern cvar_t sv_fixedframeratesingleplayer;
 extern cvar_t sv_freezenonclients;
 extern cvar_t sv_friction;
 extern cvar_t sv_gameplayfix_blowupfallenzombies;
+extern cvar_t sv_gameplayfix_delayprojectiles;
 extern cvar_t sv_gameplayfix_droptofloorstartsolid;
+extern cvar_t sv_gameplayfix_droptofloorstartsolid_nudgetocorrect;
+extern cvar_t sv_gameplayfix_easierwaterjump;
 extern cvar_t sv_gameplayfix_findradiusdistancetobox;
 extern cvar_t sv_gameplayfix_grenadebouncedownslopes;
+extern cvar_t sv_gameplayfix_multiplethinksperframe;
 extern cvar_t sv_gameplayfix_noairborncorpse;
-extern cvar_t sv_gameplayfix_qwplayerphysics;
 extern cvar_t sv_gameplayfix_setmodelrealbox;
 extern cvar_t sv_gameplayfix_stepdown;
 extern cvar_t sv_gameplayfix_stepwhilejumping;
@@ -393,6 +426,7 @@ void SV_Init (void);
 void SV_StartParticle (vec3_t org, vec3_t dir, int color, int count);
 void SV_StartEffect (vec3_t org, int modelindex, int startframe, int framecount, int framerate);
 void SV_StartSound (prvm_edict_t *entity, int channel, const char *sample, int volume, float attenuation);
+void SV_StartPointSound (vec3_t origin, const char *sample, int volume, float attenuation);
 
 void SV_ConnectClient (int clientnum, netconn_t *netconnection);
 void SV_DropClient (qboolean crash);
@@ -434,12 +468,16 @@ qboolean SV_movestep (prvm_edict_t *ent, vec3_t move, qboolean relink, qboolean
 // if touchtriggers, calls prog functions for the intersected triggers
 void SV_LinkEdict (prvm_edict_t *ent, qboolean touch_triggers);
 
+// move an entity that is stuck by small amounts in various directions to try to nudge it back into the collision hull
+// returns true if it found a better place
+qboolean SV_UnstickEntity (prvm_edict_t *ent);
+
 // calculates hitsupercontentsmask for a generic qc entity
 int SV_GenericHitSuperContentsMask(const prvm_edict_t *edict);
 // traces a box move against worldmodel and all entities in the specified area
 trace_t SV_Move(const vec3_t start, const vec3_t mins, const vec3_t maxs, const vec3_t end, int type, prvm_edict_t *passedict, int hitsupercontentsmask);
 
-#define SV_PointSuperContents(point) (SV_Move((point), vec3_origin, vec3_origin, (point), sv_gameplayfix_swiminbmodels.integer ? MOVE_NOMONSTERS : MOVE_WORLDONLY, NULL, 0).startsupercontents)
+int SV_PointSuperContents(const vec3_t point);
 
 void SV_FlushBroadcastMessages(void);
 void SV_WriteClientdataToMessage (client_t *client, prvm_edict_t *ent, sizebuf_t *msg, int *stats);
@@ -457,5 +495,7 @@ void SV_SetupVM(void);
 void SV_VM_Begin(void);
 void SV_VM_End(void);
 
+const char *Host_TimingReport(); // for output in Host_Status_f
+
 #endif
 
