@@ -353,16 +353,7 @@ qboolean OGG_OpenLibrary (void)
 	// Load the DLLs
 	// We need to load both by hand because some OSes seem to not load
 	// the vorbis DLL automatically when loading the VorbisFile DLL
-	if (! Sys_LoadLibrary (dllnames_vo, &vo_dll, vorbisfuncs) ||
-		! Sys_LoadLibrary (dllnames_vf, &vf_dll, vorbisfilefuncs))
-	{
-		Sys_UnloadLibrary (&vo_dll);
-		Con_Printf ("Ogg Vorbis support disabled\n");
-		return false;
-	}
-
-	Con_Printf ("Ogg Vorbis support enabled\n");
-	return true;
+	return Sys_LoadLibrary (dllnames_vo, &vo_dll, vorbisfuncs) && Sys_LoadLibrary (dllnames_vf, &vf_dll, vorbisfilefuncs);
 }
 
 
@@ -402,6 +393,8 @@ typedef struct
 	unsigned char	*file;
 	size_t			filesize;
 	snd_format_t	format;
+	unsigned int	total_length;
+	char			name[128];
 } ogg_stream_persfx_t;
 
 // Per-channel data structure
@@ -422,20 +415,15 @@ static const ov_callbacks callbacks = {ovcb_read, ovcb_seek, ovcb_close, ovcb_te
 OGG_FetchSound
 ====================
 */
-static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, unsigned int nbsampleframes)
+static const snd_buffer_t* OGG_FetchSound (void *sfxfetcher, void **chfetcherpointer, unsigned int *start, unsigned int nbsampleframes)
 {
-	ogg_stream_perchannel_t* per_ch;
-	sfx_t* sfx;
-	ogg_stream_persfx_t* per_sfx;
+	ogg_stream_perchannel_t* per_ch = (ogg_stream_perchannel_t *)*chfetcherpointer;
+	ogg_stream_persfx_t* per_sfx = (ogg_stream_persfx_t *)sfxfetcher;
 	snd_buffer_t* sb;
 	int newlength, done, ret, bigendian;
 	unsigned int real_start;
 	unsigned int factor;
 
-	per_ch = (ogg_stream_perchannel_t *)ch->fetcher_data;
-	sfx = ch->sfx;
-	per_sfx = (ogg_stream_persfx_t *)sfx->fetcher_data;
-
 	// If there's no fetcher structure attached to the channel yet
 	if (per_ch == NULL)
 	{
@@ -449,7 +437,6 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 		buff_len = STREAM_BUFFER_SIZE(&sb_format);
 		memsize = sizeof (*per_ch) - sizeof (per_ch->sb.samples) + buff_len;
 		per_ch = (ogg_stream_perchannel_t *)Mem_Alloc (snd_mempool, memsize);
-		sfx->memsize += memsize;
 
 		// Open it with the VorbisFile API
 		per_ch->ov_decode.buffer = per_sfx->file;
@@ -457,7 +444,7 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 		per_ch->ov_decode.buffsize = per_sfx->filesize;
 		if (qov_open_callbacks (&per_ch->ov_decode, &per_ch->vf, NULL, 0, callbacks) < 0)
 		{
-			Con_Printf("error while reading Ogg Vorbis stream \"%s\"\n", sfx->name);
+			Con_Printf("error while reading Ogg Vorbis stream \"%s\"\n", per_sfx->name);
 			Mem_Free (per_ch);
 			return NULL;
 		}
@@ -468,7 +455,7 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 		per_ch->sb.nbframes = 0;
 		per_ch->sb.maxframes = buff_len / (per_ch->sb.format.channels * per_ch->sb.format.width);
 
-		ch->fetcher_data = per_ch;
+		*chfetcherpointer = per_ch;
 	}
 
 	real_start = *start;
@@ -499,10 +486,10 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 		ogg_int64_t ogg_start;
 		int err;
 
-		if (real_start > (unsigned int)sfx->total_length)
+		if (real_start > (unsigned int)per_sfx->total_length)
 		{
 			Con_Printf ("OGG_FetchSound: asked for a start position after the end of the sfx! (%u > %u)\n",
-						real_start, sfx->total_length);
+						real_start, per_sfx->total_length);
 			return NULL;
 		}
 
@@ -546,6 +533,8 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 		return NULL;
 	}
 	newlength = per_sfx->format.speed * factor;  // -> 1 sec of sound before resampling
+	if(newlength > (int)sizeof(resampling_buffer))
+		newlength = sizeof(resampling_buffer);
 
 	// Decompress in the resampling_buffer
 #if BYTE_ORDER == BIG_ENDIAN
@@ -569,23 +558,16 @@ static const snd_buffer_t* OGG_FetchSound (channel_t* ch, unsigned int* start, u
 OGG_FetchEnd
 ====================
 */
-static void OGG_FetchEnd (channel_t* ch)
+static void OGG_FetchEnd (void *chfetcherdata)
 {
-	ogg_stream_perchannel_t* per_ch;
+	ogg_stream_perchannel_t* per_ch = (ogg_stream_perchannel_t *)chfetcherdata;
 
-	per_ch = (ogg_stream_perchannel_t *)ch->fetcher_data;
 	if (per_ch != NULL)
 	{
-		size_t buff_len;
-
 		// Free the ogg vorbis decoder
 		qov_clear (&per_ch->vf);
 
-		buff_len = per_ch->sb.maxframes * per_ch->sb.format.channels * per_ch->sb.format.width;
-		ch->sfx->memsize -= sizeof (*per_ch) - sizeof (per_ch->sb.samples) + buff_len;
-
 		Mem_Free (per_ch);
-		ch->fetcher_data = NULL;
 	}
 }
 
@@ -595,20 +577,15 @@ static void OGG_FetchEnd (channel_t* ch)
 OGG_FreeSfx
 ====================
 */
-static void OGG_FreeSfx (sfx_t* sfx)
+static void OGG_FreeSfx (void *sfxfetcherdata)
 {
-	ogg_stream_persfx_t* per_sfx = (ogg_stream_persfx_t *)sfx->fetcher_data;
+	ogg_stream_persfx_t* per_sfx = (ogg_stream_persfx_t *)sfxfetcherdata;
 
 	// Free the Ogg Vorbis file
 	Mem_Free(per_sfx->file);
-	sfx->memsize -= per_sfx->filesize;
 
 	// Free the stream structure
 	Mem_Free(per_sfx);
-	sfx->memsize -= sizeof (*per_sfx);
-
-	sfx->fetcher_data = NULL;
-	sfx->fetcher = NULL;
 }
 
 
@@ -636,13 +613,15 @@ Load an Ogg Vorbis file into memory
 qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 {
 	unsigned char *data;
-	const char *loopcomment;
+	const char *thiscomment;
 	fs_offset_t filesize;
 	ov_decode_t ov_decode;
 	OggVorbis_File vf;
 	vorbis_info *vi;
 	vorbis_comment *vc;
 	ogg_int64_t len, buff_len;
+	double peak = 0.0;
+	double gaindb = 0.0;
 
 	if (!vf_dll)
 		return false;
@@ -656,7 +635,8 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 	if (data == NULL)
 		return false;
 
-	Con_DPrintf ("Loading Ogg Vorbis file \"%s\"\n", filename);
+	if (developer_loading.integer >= 2)
+		Con_Printf ("Loading Ogg Vorbis file \"%s\"\n", filename);
 
 	// Open it with the VorbisFile API
 	ov_decode.buffer = data;
@@ -688,8 +668,10 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 	{
 		ogg_stream_persfx_t* per_sfx;
 
-		Con_DPrintf ("\"%s\" will be streamed\n", filename);
+		if (developer_loading.integer >= 2)
+			Con_Printf ("Ogg sound file \"%s\" will be streamed\n", filename);
 		per_sfx = (ogg_stream_persfx_t *)Mem_Alloc (snd_mempool, sizeof (*per_sfx));
+		strlcpy(per_sfx->name, sfx->name, sizeof(per_sfx->name));
 		sfx->memsize += sizeof (*per_sfx);
 		per_sfx->file = data;
 		per_sfx->filesize = filesize;
@@ -702,14 +684,20 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 		sfx->fetcher_data = per_sfx;
 		sfx->fetcher = &ogg_fetcher;
 		sfx->flags |= SFXFLAG_STREAMED;
-		sfx->total_length = (int)((size_t)len / (per_sfx->format.channels * 2) * ((double)snd_renderbuffer->format.speed / per_sfx->format.speed));
+		per_sfx->total_length = sfx->total_length = (int)((size_t)len / (per_sfx->format.channels * 2) * ((double)snd_renderbuffer->format.speed / per_sfx->format.speed));
 		sfx->loopstart = sfx->total_length;
 		vc = qov_comment(&vf, -1);
 		if(vc)
 		{
-			loopcomment = qvorbis_comment_query(vc, "LOOP_START", 0);
-			if(loopcomment)
-				sfx->loopstart = bound(0, (unsigned int) (atof(loopcomment) * (double)snd_renderbuffer->format.speed / (double)per_sfx->format.speed), sfx->total_length);
+			thiscomment = qvorbis_comment_query(vc, "LOOP_START", 0);
+			if(thiscomment)
+				sfx->loopstart = bound(0, (unsigned int) (atof(thiscomment) * (double)snd_renderbuffer->format.speed / (double)per_sfx->format.speed), sfx->total_length);
+			thiscomment = qvorbis_comment_query(vc, "REPLAYGAIN_TRACK_PEAK", 0);
+			if(thiscomment)
+				peak = atof(thiscomment);
+			thiscomment = qvorbis_comment_query(vc, "REPLAYGAIN_TRACK_GAIN", 0);
+			if(thiscomment)
+				gaindb = atof(thiscomment);
 		}
 	}
 	else
@@ -721,7 +709,8 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 		snd_buffer_t *sb;
 		snd_format_t ogg_format;
 
-		Con_DPrintf ("\"%s\" will be cached\n", filename);
+		if (developer_loading.integer >= 2)
+			Con_Printf ("Ogg sound file \"%s\" will be cached\n", filename);
 
 		// Decode it
 		buff = (char *)Mem_Alloc (snd_mempool, (int)len);
@@ -759,9 +748,15 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 		vc = qov_comment(&vf, -1);
 		if(vc)
 		{
-			loopcomment = qvorbis_comment_query(vc, "LOOP_START", 0);
-			if(loopcomment)
-				sfx->loopstart = bound(0, (unsigned int) (atoi(loopcomment) * (double)snd_renderbuffer->format.speed / (double)sb->format.speed), sfx->total_length);
+			thiscomment = qvorbis_comment_query(vc, "LOOP_START", 0);
+			if(thiscomment)
+				sfx->loopstart = bound(0, (unsigned int) (atoi(thiscomment) * (double)snd_renderbuffer->format.speed / (double)sb->format.speed), sfx->total_length);
+			thiscomment = qvorbis_comment_query(vc, "REPLAYGAIN_TRACK_PEAK", 0);
+			if(thiscomment)
+				peak = atof(thiscomment);
+			thiscomment = qvorbis_comment_query(vc, "REPLAYGAIN_TRACK_GAIN", 0);
+			if(thiscomment)
+				gaindb = atof(thiscomment);
 		}
 
 		qov_clear (&vf);
@@ -769,5 +764,13 @@ qboolean OGG_LoadVorbisFile (const char *filename, sfx_t *sfx)
 		Mem_Free (buff);
 	}
 
+	if(peak)
+	{
+		sfx->volume_mult = min(1 / peak, exp(gaindb * 0.05 * log(10)));
+		sfx->volume_peak = peak;
+		if (developer_loading.integer >= 2)
+			Con_Printf ("Ogg sound file \"%s\" uses ReplayGain (gain %f, peak %f)\n", filename, sfx->volume_mult, sfx->volume_peak);
+	}
+
 	return true;
 }
