@@ -107,7 +107,7 @@ trace_t SV_Move(const vec3_t start, const vec3_t mins, const vec3_t maxs, const
 	// matrices to transform into/out of other entity's space
 	matrix4x4_t matrix, imatrix;
 	// model of other entity
-	model_t *model;
+	dp_model_t *model;
 	// list of entities to test for collisions
 	int numtouchedicts;
 	prvm_edict_t *touchedicts[MAX_EDICTS];
@@ -264,7 +264,7 @@ int SV_PointSuperContents(const vec3_t point)
 	// matrices to transform into/out of other entity's space
 	matrix4x4_t matrix, imatrix;
 	// model of other entity
-	model_t *model;
+	dp_model_t *model;
 	unsigned int modelindex;
 	int frame;
 	// list of entities to test for collisions
@@ -378,7 +378,7 @@ SV_LinkEdict
 */
 void SV_LinkEdict (prvm_edict_t *ent, qboolean touch_triggers)
 {
-	model_t *model;
+	dp_model_t *model;
 	vec3_t mins, maxs;
 
 	if (ent == prog->edicts)
@@ -1062,7 +1062,7 @@ void SV_PushMove (prvm_edict_t *pusher, float movetime)
 	int num_moved;
 	int numcheckentities;
 	static prvm_edict_t *checkentities[MAX_EDICTS];
-	model_t *pushermodel;
+	dp_model_t *pushermodel;
 	trace_t trace;
 	matrix4x4_t pusherfinalmatrix, pusherfinalimatrix;
 	unsigned short moved_edicts[MAX_EDICTS];
@@ -1991,6 +1991,13 @@ will fall if the floor is pulled out from under them.
 void SV_Physics_Step (prvm_edict_t *ent)
 {
 	int flags = (int)ent->fields.server->flags;
+
+	// DRESK
+	// Backup Velocity in the event that movetypesteplandevent is called,
+	// to provide a parameter with the entity's velocity at impact.
+	prvm_eval_t *movetypesteplandevent;
+	vec3_t backupVelocity;
+	VectorCopy(ent->fields.server->velocity, backupVelocity);
 	// don't fall at all if fly/swim
 	if (!(flags & (FL_FLY | FL_SWIM)))
 	{
@@ -2019,8 +2026,28 @@ void SV_Physics_Step (prvm_edict_t *ent)
 			SV_LinkEdict(ent, true);
 
 			// just hit ground
-			if (hitsound && (int)ent->fields.server->flags & FL_ONGROUND && sv_sound_land.string)
-				SV_StartSound(ent, 0, sv_sound_land.string, 255, 1);
+			if (hitsound && (int)ent->fields.server->flags & FL_ONGROUND)
+			{
+				// DRESK - Check for Entity Land Event Function
+				movetypesteplandevent = PRVM_EDICTFIELDVALUE(ent, prog->fieldoffsets.movetypesteplandevent);
+
+				if(movetypesteplandevent->function)
+				{ // Valid Function; Execute
+					// Prepare Parameters
+						// Assign Velocity at Impact
+						PRVM_G_VECTOR(OFS_PARM0)[0] = backupVelocity[0];
+						PRVM_G_VECTOR(OFS_PARM0)[1] = backupVelocity[1];
+						PRVM_G_VECTOR(OFS_PARM0)[2] = backupVelocity[2];
+						// Assign Self
+						prog->globals.server->self = PRVM_EDICT_TO_PROG(ent);
+					// Execute VM Function
+					PRVM_ExecuteProgram(movetypesteplandevent->function, "movetypesteplandevent: NULL function");
+				}
+				else
+				// Check for Engine Landing Sound
+				if(sv_sound_land.string)
+					SV_StartSound(ent, 0, sv_sound_land.string, 255, 1);
+			}
 			ent->priv.server->waterposition_forceupdate = true;
 		}
 	}
