@@ -1,3 +1,4 @@
+#include "quakedef.h"
 
 #ifdef WIN32
 #include <io.h>
@@ -10,8 +11,6 @@
 
 #include <signal.h>
 
-#include "quakedef.h"
-
 
 #ifdef WIN32
 cvar_t sys_usetimegettime = {CVAR_SAVE, "sys_usetimegettime", "1", "use windows timeGetTime function (which has issues on some motherboards) for timing rather than QueryPerformanceCounter timer (which has issues on multicore/multiprocessor machines and processors which are designed to conserve power)"};
@@ -217,12 +216,8 @@ char *Sys_ConsoleInput(void)
 void Sys_Sleep(int microseconds)
 {
 #ifdef WIN32
-	if (microseconds < 1000)
-		microseconds = 1000;
 	Sleep(microseconds / 1000);
 #else
-	if (microseconds < 1)
-		microseconds = 1;
 	usleep(microseconds);
 #endif
 }
