@@ -1,3 +1,4 @@
+#include "quakedef.h"
 
 #ifdef WIN32
 #include <io.h>
@@ -10,8 +11,6 @@
 
 #include <signal.h>
 
-#include "quakedef.h"
-
 #include <SDL.h>
 
 // =======================================================================
@@ -157,8 +156,6 @@ char *Sys_ConsoleInput(void)
 
 void Sys_Sleep(int microseconds)
 {
-	if (microseconds < 1000)
-		microseconds = 1000;
 	SDL_Delay(microseconds / 1000);
 }
 
