@@ -703,19 +703,10 @@ int VID_InitMode(int fullscreen, int width, int height, int bpp, int refreshrate
 	if ((qglGetString = (const GLubyte* (GLAPIENTRY *)(GLenum name))GL_GetProcAddress("glGetString")) == NULL)
 		Sys_Error("glGetString not found in %s", gl_driver);
 
-	gl_renderer = (const char *)qglGetString(GL_RENDERER);
-	gl_vendor = (const char *)qglGetString(GL_VENDOR);
-	gl_version = (const char *)qglGetString(GL_VERSION);
-	gl_extensions = (const char *)qglGetString(GL_EXTENSIONS);
+	gl_platformextensions = "";
 	gl_platform = "AGL";
 	gl_videosyncavailable = true;
 
-	Con_DPrintf("GL_VENDOR: %s\n", gl_vendor);
-	Con_DPrintf("GL_RENDERER: %s\n", gl_renderer);
-	Con_DPrintf("GL_VERSION: %s\n", gl_version);
-	Con_DPrintf("GL_EXTENSIONS: %s\n", gl_extensions);
-	Con_DPrintf("%s_EXTENSIONS: %s\n", gl_platform, gl_platformextensions);
-
 	multithreadedgl = false;
 	vid_isfullscreen = fullscreen;
 	vid_usingmouse = false;
