@@ -89,7 +89,6 @@ Atom wm_delete_window_atom;
 
 static qboolean mouse_avail = true;
 static qboolean vid_usingmouse = false;
-static qboolean vid_usingdgamouse = false;
 static qboolean vid_usingvsync = false;
 static qboolean vid_usevsync = false;
 static qboolean vid_x11_hardwaregammasupported = false;
@@ -100,6 +99,7 @@ static int p_mouse_x, p_mouse_y;
 
 #if !defined(__APPLE__) && !defined(SUNOS)
 cvar_t vid_dgamouse = {CVAR_SAVE, "vid_dgamouse", "1", "make use of DGA mouse input"};
+static qboolean vid_usingdgamouse = false;
 #endif
 
 qboolean vidmode_ext = false;
@@ -289,7 +289,9 @@ static void IN_Activate (qboolean grab)
 			mouse_x = mouse_y = 0;
 			cl_ignoremousemoves = 2;
 			vid_usingmouse = true;
+#if !defined(__APPLE__) && !defined(SUNOS)
 			vid_usingdgamouse = !!vid_dgamouse.integer;
+#endif
 		}
 	}
 	else
@@ -855,19 +857,10 @@ int VID_InitMode(int fullscreen, int width, int height, int bpp, int refreshrate
 		return false;
 	}
 
-	gl_renderer = (const char *)qglGetString(GL_RENDERER);
-	gl_vendor = (const char *)qglGetString(GL_VENDOR);
-	gl_version = (const char *)qglGetString(GL_VERSION);
 	gl_extensions = (const char *)qglGetString(GL_EXTENSIONS);
 	gl_platform = "GLX";
 	gl_platformextensions = qglXQueryExtensionsString(vidx11_display, vidx11_screen);
 
-	Con_DPrintf("GL_VENDOR: %s\n", gl_vendor);
-	Con_DPrintf("GL_RENDERER: %s\n", gl_renderer);
-	Con_DPrintf("GL_VERSION: %s\n", gl_version);
-	Con_DPrintf("GL_EXTENSIONS: %s\n", gl_extensions);
-	Con_DPrintf("%s_EXTENSIONS: %s\n", gl_platform, gl_platformextensions);
-
 	gl_videosyncavailable = false;
 
 // COMMANDLINEOPTION: Linux GLX: -nogetprocaddress disables GLX_ARB_get_proc_address (not required, more formal method of getting extension functions)
