@@ -732,21 +732,9 @@ int VID_InitMode(int fullscreen, int width, int height, int bpp, int refreshrate
 	// enable key repeat since everyone expects it
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 
-	gl_renderer = (const char *)qglGetString(GL_RENDERER);
-	gl_vendor = (const char *)qglGetString(GL_VENDOR);
-	gl_version = (const char *)qglGetString(GL_VERSION);
-	gl_extensions = (const char *)qglGetString(GL_EXTENSIONS);
 	gl_platform = "SDL";
-	// Knghtbrd: should assign platform-specific extensions here
-	//TODO: maybe ;)
 	gl_platformextensions = "";
-	gl_videosyncavailable = false;
-
-	Con_DPrintf("GL_VENDOR: %s\n", gl_vendor);
-	Con_DPrintf("GL_RENDERER: %s\n", gl_renderer);
-	Con_DPrintf("GL_VERSION: %s\n", gl_version);
-	Con_DPrintf("GL_EXTENSIONS: %s\n", gl_extensions);
-	Con_DPrintf("%s_EXTENSIONS: %s\n", gl_platform, gl_platformextensions);
+	gl_videosyncavailable = true;
 
 	GL_Init();
 
@@ -782,6 +770,11 @@ void VID_Shutdown (void)
 
 	IN_Activate(false);
 	SDL_QuitSubSystem(SDL_INIT_VIDEO);
+
+	gl_driver[0] = 0;
+	gl_extensions = "";
+	gl_platform = "";
+	gl_platformextensions = "";
 }
 
 int VID_SetGamma (unsigned short *ramps, int rampsize)
