@@ -527,7 +527,7 @@ LONG WINAPI MainWndProc (HWND hWnd, UINT uMsg, WPARAM  wParam, LPARAM lParam)
 			GetKeyboardState (state);
 			// alt/ctrl/shift tend to produce funky ToAscii values,
 			// and if it's not a single character we don't know care about it
-			charlength = ToAscii (wParam, lParam >> 16, state, (unsigned short *)asciichar, 0);
+			charlength = ToAscii (wParam, lParam >> 16, state, (LPWORD)asciichar, 0);
 			if (vkey == K_ALT || vkey == K_CTRL || vkey == K_SHIFT || charlength == 0)
 				asciichar[0] = 0;
 			else if( charlength == 2 ) {
@@ -1095,8 +1095,12 @@ int VID_InitMode (int fullscreen, int width, int height, int bpp, int refreshrat
 
 	pixelformat = 0;
 	newpixelformat = 0;
-	for (windowpass = 0;windowpass < 2;windowpass++)
+	// start out at the final windowpass if samples is 1 as it's the only feature we need extended pixel formats for
+	for (windowpass = samples == 1;windowpass < 2;windowpass++)
 	{
+		gl_extensions = "";
+		gl_platformextensions = "";
+
 		mainwindow = CreateWindowEx (ExWindowStyle, "DarkPlacesWindowClass", gamename, WindowStyle, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, NULL, NULL, global_hInstance, NULL);
 		if (!mainwindow)
 		{
@@ -1154,15 +1158,17 @@ int VID_InitMode (int fullscreen, int width, int height, int bpp, int refreshrat
 		if ((qwglGetExtensionsStringARB = (const char *(WINAPI *)(HDC hdc))GL_GetProcAddress("wglGetExtensionsStringARB")) == NULL)
 			Con_Print("wglGetExtensionsStringARB not found\n");
 
-		gl_renderer = qglGetString(GL_RENDERER);
-		gl_vendor = qglGetString(GL_VENDOR);
-		gl_version = qglGetString(GL_VERSION);
-		gl_extensions = qglGetString(GL_EXTENSIONS);
+		gl_extensions = (const char *)qglGetString(GL_EXTENSIONS);
 		gl_platform = "WGL";
 		gl_platformextensions = "";
 
 		if (qwglGetExtensionsStringARB)
-			gl_platformextensions = qwglGetExtensionsStringARB(baseDC);
+			gl_platformextensions = (const char *)qwglGetExtensionsStringARB(baseDC);
+
+		if (!gl_extensions)
+			gl_extensions = "";
+		if (!gl_platformextensions)
+			gl_platformextensions = "";
 
 		// now some nice Windows pain:
 		// we have created a window, we needed one to find out if there are
@@ -1187,12 +1193,6 @@ int VID_InitMode (int fullscreen, int width, int height, int bpp, int refreshrat
 		}
 	}
 
-	Con_DPrintf("GL_VENDOR: %s\n", gl_vendor);
-	Con_DPrintf("GL_RENDERER: %s\n", gl_renderer);
-	Con_DPrintf("GL_VERSION: %s\n", gl_version);
-	Con_DPrintf("GL_EXTENSIONS: %s\n", gl_extensions);
-	Con_DPrintf("%s_EXTENSIONS: %s\n", gl_platform, gl_platformextensions);
-
 	/*
 	if (!fullscreen)
 		SetWindowPos (mainwindow, NULL, CenterX, CenterY, 0, 0,SWP_NOSIZE | SWP_NOZORDER | SWP_SHOWWINDOW | SWP_DRAWFRAME);
