@@ -22,6 +22,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "quakedef.h"
 #include "cl_collision.h"
 
+void CL_VM_UpdateDmgGlobals (int dmg_take, int dmg_save, vec3_t dmg_origin);
+
 /*
 
 The view is allowed to move slightly from it's true position for bobbing,
@@ -68,6 +70,7 @@ cvar_t cl_stairsmoothspeed = {CVAR_SAVE, "cl_stairsmoothspeed", "160", "how fast
 cvar_t chase_back = {CVAR_SAVE, "chase_back", "48", "chase cam distance from the player"};
 cvar_t chase_up = {CVAR_SAVE, "chase_up", "24", "chase cam distance from the player"};
 cvar_t chase_active = {CVAR_SAVE, "chase_active", "0", "enables chase cam"};
+cvar_t chase_overhead = {CVAR_SAVE, "chase_overhead", "0", "chase cam looks straight down if this is not zero"};
 // GAME_GOODVSBAD2
 cvar_t chase_stevie = {0, "chase_stevie", "0", "chase cam view from above (used only by GoodVsBad2)"};
 
@@ -226,6 +229,9 @@ void V_ParseDamage (void)
 	blood = MSG_ReadByte ();
 	MSG_ReadVector(from, cls.protocol);
 
+	// Send the Dmg Globals to CSQC
+	CL_VM_UpdateDmgGlobals(blood, armor, from);
+
 	count = blood*0.5 + armor*0.5;
 	if (count < 10)
 		count = 10;
@@ -319,6 +325,10 @@ V_CalcRefdef
 
 ==================
 */
+#if 0
+static vec3_t eyeboxmins = {-16, -16, -24};
+static vec3_t eyeboxmaxs = { 16,  16,  32};
+#endif
 void V_CalcRefdef (void)
 {
 	entity_t *ent;
@@ -326,7 +336,7 @@ void V_CalcRefdef (void)
 	trace_t trace;
 	VectorClear(gunorg);
 	viewmodelmatrix = identitymatrix;
-	r_view.matrix = identitymatrix;
+	r_refdef.view.matrix = identitymatrix;
 	if (cls.state == ca_connected && cls.signon == SIGNONS)
 	{
 		// ent is the view entity (visible when out of body)
@@ -348,13 +358,13 @@ void V_CalcRefdef (void)
 		{
 			// entity is a fixed camera, just copy the matrix
 			if (cls.protocol == PROTOCOL_QUAKEWORLD)
-				Matrix4x4_CreateFromQuakeEntity(&r_view.matrix, cl.qw_intermission_origin[0], cl.qw_intermission_origin[1], cl.qw_intermission_origin[2], cl.qw_intermission_angles[0], cl.qw_intermission_angles[1], cl.qw_intermission_angles[2], 1);
+				Matrix4x4_CreateFromQuakeEntity(&r_refdef.view.matrix, cl.qw_intermission_origin[0], cl.qw_intermission_origin[1], cl.qw_intermission_origin[2], cl.qw_intermission_angles[0], cl.qw_intermission_angles[1], cl.qw_intermission_angles[2], 1);
 			else
 			{
-				r_view.matrix = ent->render.matrix;
-				Matrix4x4_AdjustOrigin(&r_view.matrix, 0, 0, cl.stats[STAT_VIEWHEIGHT]);
+				r_refdef.view.matrix = ent->render.matrix;
+				Matrix4x4_AdjustOrigin(&r_refdef.view.matrix, 0, 0, cl.stats[STAT_VIEWHEIGHT]);
 			}
-			viewmodelmatrix = r_view.matrix;
+			viewmodelmatrix = r_refdef.view.matrix;
 		}
 		else
 		{
@@ -378,31 +388,72 @@ void V_CalcRefdef (void)
 				// observing entity from third person
 				vec_t camback, camup, dist, forward[3], chase_dest[3];
 
-				camback = bound(0, chase_back.value, 128);
-				if (chase_back.value != camback)
-					Cvar_SetValueQuick(&chase_back, camback);
-				camup = bound(-48, chase_up.value, 96);
-				if (chase_up.value != camup)
-					Cvar_SetValueQuick(&chase_up, camup);
+				camback = chase_back.value;
+				camup = chase_up.value;
 
 				// this + 22 is to match view_ofs for compatibility with older versions
 				camup += 22;
 
-				if (gamemode == GAME_GOODVSBAD2 && chase_stevie.integer)
-				{
-					// look straight down from high above
-					viewangles[0] = 90;
-					camback = 2048;
-				}
 				AngleVectors(viewangles, forward, NULL, NULL);
 
-				// trace a little further so it hits a surface more consistently (to avoid 'snapping' on the edge of the range)
-				dist = -camback - 8;
-				chase_dest[0] = vieworg[0] + forward[0] * dist;
-				chase_dest[1] = vieworg[1] + forward[1] * dist;
-				chase_dest[2] = vieworg[2] + forward[2] * dist + camup;
-				trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
-				VectorMAMAM(1, trace.endpos, 8, forward, 4, trace.plane.normal, vieworg);
+				if (chase_overhead.integer)
+				{
+#if 1
+					vec3_t offset;
+					vec3_t bestvieworg;
+#endif
+					vec3_t up;
+					viewangles[PITCH] = 0;
+					AngleVectors(viewangles, forward, NULL, up);
+					// trace a little further so it hits a surface more consistently (to avoid 'snapping' on the edge of the range)
+					chase_dest[0] = vieworg[0] - forward[0] * camback + up[0] * camup;
+					chase_dest[1] = vieworg[1] - forward[1] * camback + up[1] * camup;
+					chase_dest[2] = vieworg[2] - forward[2] * camback + up[2] * camup;
+#if 0
+					//trace = CL_Move(vieworg, eyeboxmins, eyeboxmaxs, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
+					trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
+					VectorCopy(trace.endpos, vieworg);
+					vieworg[2] -= 8;
+#else
+					trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
+					VectorCopy(trace.endpos, bestvieworg);
+					offset[2] = 0;
+					for (offset[0] = -16;offset[0] <= 16;offset[0] += 8)
+					{
+						for (offset[1] = -16;offset[1] <= 16;offset[1] += 8)
+						{
+							AngleVectors(viewangles, NULL, NULL, up);
+							chase_dest[0] = vieworg[0] - forward[0] * camback + up[0] * camup + offset[0];
+							chase_dest[1] = vieworg[1] - forward[1] * camback + up[1] * camup + offset[1];
+							chase_dest[2] = vieworg[2] - forward[2] * camback + up[2] * camup + offset[2];
+							trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
+							if (bestvieworg[2] > trace.endpos[2])
+								bestvieworg[2] = trace.endpos[2];
+						}
+					}
+					bestvieworg[2] -= 8;
+					VectorCopy(bestvieworg, vieworg);
+#endif
+					viewangles[PITCH] = 90;
+				}
+				else
+				{
+					if (gamemode == GAME_GOODVSBAD2 && chase_stevie.integer)
+					{
+						// look straight down from high above
+						viewangles[PITCH] = 90;
+						camback = 2048;
+						VectorSet(forward, 0, 0, -1);
+					}
+
+					// trace a little further so it hits a surface more consistently (to avoid 'snapping' on the edge of the range)
+					dist = -camback - 8;
+					chase_dest[0] = vieworg[0] + forward[0] * dist;
+					chase_dest[1] = vieworg[1] + forward[1] * dist;
+					chase_dest[2] = vieworg[2] + forward[2] * dist + camup;
+					trace = CL_Move(vieworg, vec3_origin, vec3_origin, chase_dest, MOVE_NOMONSTERS, NULL, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_SKY, true, false, NULL, false);
+					VectorMAMAM(1, trace.endpos, 8, forward, 4, trace.plane.normal, vieworg);
+				}
 			}
 			else
 			{
@@ -491,9 +542,9 @@ void V_CalcRefdef (void)
 			}
 			// calculate a view matrix for rendering the scene
 			if (v_idlescale.value)
-				Matrix4x4_CreateFromQuakeEntity(&r_view.matrix, vieworg[0], vieworg[1], vieworg[2], viewangles[0] + v_idlescale.value * sin(cl.time*v_ipitch_cycle.value) * v_ipitch_level.value, viewangles[1] + v_idlescale.value * sin(cl.time*v_iyaw_cycle.value) * v_iyaw_level.value, viewangles[2] + v_idlescale.value * sin(cl.time*v_iroll_cycle.value) * v_iroll_level.value, 1);
+				Matrix4x4_CreateFromQuakeEntity(&r_refdef.view.matrix, vieworg[0], vieworg[1], vieworg[2], viewangles[0] + v_idlescale.value * sin(cl.time*v_ipitch_cycle.value) * v_ipitch_level.value, viewangles[1] + v_idlescale.value * sin(cl.time*v_iyaw_cycle.value) * v_iyaw_level.value, viewangles[2] + v_idlescale.value * sin(cl.time*v_iroll_cycle.value) * v_iroll_level.value, 1);
 			else
-				Matrix4x4_CreateFromQuakeEntity(&r_view.matrix, vieworg[0], vieworg[1], vieworg[2], viewangles[0], viewangles[1], viewangles[2] + v_idlescale.value * sin(cl.time*v_iroll_cycle.value) * v_iroll_level.value, 1);
+				Matrix4x4_CreateFromQuakeEntity(&r_refdef.view.matrix, vieworg[0], vieworg[1], vieworg[2], viewangles[0], viewangles[1], viewangles[2] + v_idlescale.value * sin(cl.time*v_iroll_cycle.value) * v_iroll_level.value, 1);
 			// calculate a viewmodel matrix for use in view-attached entities
 			Matrix4x4_CreateFromQuakeEntity(&viewmodelmatrix, gunorg[0], gunorg[1], gunorg[2], viewangles[0], viewangles[1], viewangles[2], cl_viewmodel_scale.value);
 			VectorCopy(vieworg, cl.csqc_origin);
@@ -527,12 +578,12 @@ void V_CalcViewBlend(void)
 	r_refdef.viewblend[3] = 0;
 	r_refdef.frustumscale_x = 1;
 	r_refdef.frustumscale_y = 1;
-	if (cls.state == ca_connected && cls.signon == SIGNONS && gl_polyblend.value > 0)
+	if (cls.state == ca_connected && cls.signon == SIGNONS)
 	{
 		// set contents color
 		int supercontents;
 		vec3_t vieworigin;
-		Matrix4x4_OriginFromMatrix(&r_view.matrix, vieworigin);
+		Matrix4x4_OriginFromMatrix(&r_refdef.view.matrix, vieworigin);
 		supercontents = CL_PointSuperContents(vieworigin);
 		if (supercontents & SUPERCONTENTS_LIQUIDSMASK)
 		{
@@ -676,6 +727,7 @@ void V_Init (void)
 	Cvar_RegisterVariable (&chase_back);
 	Cvar_RegisterVariable (&chase_up);
 	Cvar_RegisterVariable (&chase_active);
+	Cvar_RegisterVariable (&chase_overhead);
 	if (gamemode == GAME_GOODVSBAD2)
 		Cvar_RegisterVariable (&chase_stevie);
 
